<?php

/**
 * Custom fields and filters of post
 *
 * @package CustomFieldFilterPost
 * @version 1.2
 * @author davidhs
 * @copyright 2014 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License http://creativecommons.org/licenses/by/3.0/
 */

if (!defined('SMF'))
	die('Hacking attempt...');

// Create a table.
function cffp_db_create_table($table_name, $columns, $indexes = array(), $no_prefix = true, $if_exists = 'ignore')
{
	global $db_prefix, $db_character_set;

	// Strip out the table name, we might not need it in some cases.
	$real_prefix = preg_match('~^(`?)(.+?)\\1\\.(.*?)$~', $db_prefix, $match) === 1 ? $match[3] : $db_prefix;

	// With or without the database name, the fullname looks like this.
	$full_table_name = $no_prefix ? $real_prefix . $table_name : $table_name;
	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	// Slightly easier on MySQL than the others...
	$tables = cffp_db_list_tables();
	if (in_array($full_table_name, $tables))
	{
		// This is a sad day... drop the table? If not, return false (error) by default.
		if ($if_exists == 'overwrite')
			cffp_db_drop_table($table_name);
		else
			return $if_exists == 'ignore';
	}

	// Righty - let's do the damn thing!
	$table_query = 'CREATE TABLE ' . $table_name . "\n" . '(';
	foreach ($columns as $column)
	{
		// Auto increment is easy here!
		if (!empty($column['auto']))
		{
			$default = 'auto_increment';
		}
		elseif (isset($column['default']) && $column['default'] !== null)
			$default = 'default \'' . addslashes($column['default']) . '\'';
		else
			$default = '';

		// Sort out the size... and stuff...
		$column['size'] = isset($column['size']) && is_numeric($column['size']) ? $column['size'] : null;
		list ($type, $size) = array($column['type'], $column['size']);

		// Allow unsigned integers (mysql only)
		$unsigned = in_array($type, array('int', 'tinyint', 'smallint', 'mediumint', 'bigint')) && !empty($column['unsigned']) ? 'unsigned ' : '';

		if ($size !== null)
			$type = $type . '(' . $size . ')';

		// Now just put it together!
		$table_query .= "\n\t`" .$column['name'] . '` ' . $type . ' ' . (!empty($unsigned) ? $unsigned : '') . (!empty($column['null']) ? '' : 'NOT NULL') . ' ' . $default . ',';
	}

	// Loop through the indexes next...
	foreach ($indexes as $index)
	{
		$columns = implode(',', $index['columns']);

		// Is it the primary?
		if (isset($index['type']) && $index['type'] == 'primary')
			$table_query .= "\n\t" . 'PRIMARY KEY (' . implode(',', $index['columns']) . '),';
		else
		{
			if (empty($index['name']))
				$index['name'] = implode('_', $index['columns']);
			$table_query .= "\n\t" . (isset($index['type']) && $index['type'] == 'unique' ? 'UNIQUE' : 'KEY') . ' ' . $index['name'] . ' (' . $columns . '),';
		}
	}

	// No trailing commas!
	if (substr($table_query, -1) == ',')
		$table_query = substr($table_query, 0, -1);

	$table_query .= ') ENGINE=MyISAM';
	if (!empty($db_character_set) && $db_character_set == 'utf8')
		$table_query .= ' DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci';

	// Create the table!
	db_query($table_query, __FILE__, __LINE__);
}

// Drop a table.
function cffp_db_drop_table($table_name, $no_prefix = true)
{
	global $db_prefix;

	// After stripping away the database name, this is what's left.
	$real_prefix = preg_match('~^(`?)(.+?)\\1\\.(.*?)$~', $db_prefix, $match) === 1 ? $match[3] : $db_prefix;

	// Get some aliases.
	$full_table_name = $no_prefix ? $real_prefix . $table_name : $table_name;
	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	// Does it exist?
	if (in_array($full_table_name, cffp_db_list_tables()))
	{
		db_query("DROP TABLE $table_name", __FILE__, __LINE__);

		return true;
	}

	// Otherwise do 'nout.
	return false;
}

// Add a column.
function cffp_db_add_column($table_name, $column_info, $no_prefix = true, $if_exists = 'update')
{
	global $db_prefix;

	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	// Does it exist - if so don't add it again!
	$columns = cffp_db_list_columns($table_name, false, false);
	foreach ($columns as $column)
		if ($column == $column_info['name'])
		{
			// If we're going to overwrite then use change column.
			if ($if_exists == 'update')
				return cffp_db_change_column($table_name, $column_info['name'], $column_info, false);
			else
				return false;
		}

	// Get the specifics...
	$column_info['size'] = isset($column_info['size']) && is_numeric($column_info['size']) ? $column_info['size'] : null;
	list ($type, $size) = array($column_info['type'], $column_info['size']);

	// Allow unsigned integers (mysql only).
	$unsigned = in_array($type, array('int', 'tinyint', 'smallint', 'mediumint', 'bigint')) && !empty($column_info['unsigned']) ? 'unsigned ' : '';

	if ($size !== null)
		$type = $type . '(' . $size . ')';

	// Now add the thing!
	db_query("
		ALTER TABLE $table_name
		ADD `" . $column_info['name'] . "` $type " . (!empty($unsigned) ? $unsigned : '') . (empty($column_info['null']) ? 'NOT NULL' : '') . ' ' .
			(!isset($column_info['default']) ? '' : 'default \'' . addslashes($column_info['default']) . '\'') . ' ' .
			(empty($column_info['auto']) ? '' : 'auto_increment primary key'), __FILE__, __LINE__);

	return true;
}

// Remove a column.
function cffp_db_remove_column($table_name, $column_name, $no_prefix = true)
{
	global $db_prefix;

	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	// Does it exist?
	$columns = cffp_db_list_columns($table_name, true, false);
	foreach ($columns as $column)
		if ($column['name'] == $column_name)
		{
			db_query("
				ALTER TABLE $table_name
				DROP COLUMN $column_name", __FILE__, __LINE__);

			return true;
		}

	// If here we didn't have to work - joy!
	return false;
}

// Change a column.
function cffp_db_change_column($table_name, $old_column, $column_info, $no_prefix = true)
{
	global $db_prefix;

	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	// Check it does exist!
	$columns = cffp_db_list_columns($table_name, true, false);
	$old_info = null;
	foreach ($columns as $column)
		if ($column['name'] == $old_column)
			$old_info = $column;

	// Nothing?
	if ($old_info == null)
		return false;

	// Get the right bits.
	if (!isset($column_info['name']))
		$column_info['name'] = $old_column;
	if (!isset($column_info['default']))
		$column_info['default'] = $old_info['default'];
	if (!isset($column_info['null']))
		$column_info['null'] = $old_info['null'];
	if (!isset($column_info['auto']))
		$column_info['auto'] = $old_info['auto'];
	if (!isset($column_info['type']))
		$column_info['type'] = $old_info['type'];
	if (!isset($column_info['size']) || !is_numeric($column_info['size']))
		$column_info['size'] = $old_info['size'];
	if (!isset($column_info['unsigned']) || !in_array($column_info['type'], array('int', 'tinyint', 'smallint', 'mediumint', 'bigint')))
		$column_info['unsigned'] = '';

	list ($type, $size) = array($column_info['type'], $column_info['size']);

	// Allow for unsigned integers (mysql only).
	$unsigned = in_array($type, array('int', 'tinyint', 'smallint', 'mediumint', 'bigint')) && !empty($column_info['unsigned']) ? 'unsigned ' : '';

	if ($size !== null)
		$type = $type . '(' . $size . ')';

	db_query("
		ALTER TABLE $table_name
		CHANGE COLUMN `$old_column` `" . $column_info['name'] . "` $type " . (!empty($unsigned) ? $unsigned : '') . (empty($column_info['null']) ? 'NOT NULL' : '') . ' ' .
			(!isset($column_info['default']) ? '' : 'default \'' . addslashes($column_info['default']) . '\'') . ' ' .
			(empty($column_info['auto']) ? '' : 'auto_increment'), __FILE__, __LINE__);
}

// Add an index.
function cffp_db_add_index($table_name, $index_info, $no_prefix = true, $if_exists = 'update')
{
	global $db_prefix;

	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	// No columns = no index.
	if (empty($index_info['columns']))
		return false;
	$columns = implode(',', $index_info['columns']);

	// No name - make it up!
	if (empty($index_info['name']))
	{
		// No need for primary.
		if (isset($index_info['type']) && $index_info['type'] == 'primary')
			$index_info['name'] = '';
		else
			$index_info['name'] = implode('_', $index_info['columns']);
	}
	else
		$index_info['name'] = $index_info['name'];

	// Let's get all our indexes.
	$indexes = cffp_db_list_indexes($table_name, true, false);
	// Do we already have it?
	foreach ($indexes as $index)
	{
		if ($index['name'] == $index_info['name'] || ($index['type'] == 'primary' && isset($index_info['type']) && $index_info['type'] == 'primary'))
		{
			// If we want to overwrite simply remove the current one then continue.
			if ($if_exists != 'update' || $index['type'] == 'primary')
				return false;
			else
				cffp_db_remove_index($table_name, $index_info['name'], false);
		}
	}

	// If we're here we know we don't have the index - so just add it.
	if (!empty($index_info['type']) && $index_info['type'] == 'primary')
	{
		db_query("
			ALTER TABLE $table_name
			ADD PRIMARY KEY ($columns)", __FILE__, __LINE__);
	}
	else
	{
		db_query("
			ALTER TABLE $table_name
			ADD " . (isset($index_info['type']) && $index_info['type'] == 'unique' ? 'UNIQUE' : 'INDEX') . ' ' . $index_info['name'] . " ($columns)", __FILE__, __LINE__);
	}
}

// Remove an index.
function cffp_db_remove_index($table_name, $index_name, $no_prefix = true)
{
	global $db_prefix;

	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	// Better exist!
	$indexes = cffp_db_list_indexes($table_name, true, false);

	foreach ($indexes as $index)
	{
		// If the name is primary we want the primary key!
		if ($index['type'] == 'primary' && $index_name == 'primary')
		{
			// Dropping primary key?
			db_query("
				ALTER TABLE $table_name
				DROP PRIMARY KEY", __FILE__, __LINE__);

			return true;
		}
		if ($index['name'] == $index_name)
		{
			// Drop the bugger...
			db_query("
				ALTER TABLE $table_name
				DROP INDEX $index_name", __FILE__, __LINE__);

			return true;
		}
	}

	// Not to be found ;(
	return false;
}

// Return column information for a table.
function cffp_db_list_columns($table_name, $detail = false, $no_prefix = true)
{
	global $db_prefix;

	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	$result = db_query("
		SHOW FIELDS
		FROM " . (substr($table_name, 0, 1) == '`' ? $table_name : "`$table_name`"), __FILE__, __LINE__);
	$columns = array();
	while ($row = mysql_fetch_assoc($result))
	{
		if (!$detail)
		{
			$columns[] = $row['Field'];
		}
		else
		{
			// Is there an auto_increment?
			$auto = strpos($row['Extra'], 'auto_increment') !== false ? true : false;

			// Can we split out the size?
			if (preg_match('~(.+?)\s*\((\d+)\)(?:(?:\s*)?(unsigned))?~i', $row['Type'], $matches) === 1)
			{
				$type = $matches[1];
				$size = $matches[2];
				if (!empty($matches[3]) && $matches[3] == 'unsigned')
					$unsigned = true;
			}
			else
			{
				$type = $row['Type'];
				$size = null;
			}

			$columns[$row['Field']] = array(
				'name' => $row['Field'],
				'null' => $row['Null'] != 'YES' ? false : true,
				'default' => isset($row['Default']) ? $row['Default'] : null,
				'type' => $type,
				'size' => $size,
				'auto' => $auto,
			);

			if (isset($unsigned))
			{
				$columns[$row['Field']]['unsigned'] = $unsigned;
				unset($unsigned);
			}
		}
	}
	mysql_free_result($result);

	return $columns;
}

// What about some index information?
function cffp_db_list_indexes($table_name, $detail = false, $no_prefix = true)
{
	global $db_prefix;

	if ($no_prefix)
		$table_name = $db_prefix . $table_name;

	$result = db_query("
		SHOW KEYS
		FROM " . (substr($table_name, 0, 1) == '`' ? $table_name : "`$table_name`"), __FILE__, __LINE__);
	$indexes = array();
	while ($row = mysql_fetch_assoc($result))
	{
		if (!$detail)
			$indexes[] = $row['Key_name'];
		else
		{
			// What is the type?
			if ($row['Key_name'] == 'PRIMARY')
				$type = 'primary';
			elseif (empty($row['Non_unique']))
				$type = 'unique';
			elseif (isset($row['Index_type']) && $row['Index_type'] == 'FULLTEXT')
				$type = 'fulltext';
			else
				$type = 'index';

			// This is the first column we've seen?
			if (empty($indexes[$row['Key_name']]))
			{
				$indexes[$row['Key_name']] = array(
					'name' => $row['Key_name'],
					'type' => $type,
					'columns' => array(),
				);
			}

			// Is it a partial index?
			if (!empty($row['Sub_part']))
				$indexes[$row['Key_name']]['columns'][] = $row['Column_name'] . '(' . $row['Sub_part'] . ')';
			else
				$indexes[$row['Key_name']]['columns'][] = $row['Column_name'];
		}
	}
	mysql_free_result($result);

	return $indexes;
}

// List all the tables in the database.
function cffp_db_list_tables($db = false, $filter = false)
{
	global $db_name;

	$db = $db == false ? $db_name : $db;
	$db = trim($db);
	$filter = $filter == false ? '' : " LIKE '$filter'";

	$request = db_query("
		SHOW TABLES
		FROM `" . ($db[0] == '`' ? strtr($db, array('`' => '')) : $db) . "`
		$filter", __FILE__, __LINE__);
	$tables = array();
	while ($row = mysql_fetch_row($request))
		$tables[] = $row[0];
	mysql_free_result($request);

	return $tables;
}

// Insert some data...
function cffp_db_insert($method = 'replace', $table, $columns, $data, $no_prefix = true)
{
	global $db_prefix;

	// With nothing to insert, simply return.
	if (empty($data))
		return;

	// Replace the prefix holder with the actual prefix.
	if ($no_prefix)
		$table = $db_prefix . $table;

	// Inserting data as a single row can be done as a single array.
	if (!is_array($data[array_rand($data)]))
		$data = array($data);

	// Create an array consisting of only the columns.
	$indexed_columns = array_keys($columns);

	// Here's where the variables are injected to the query.
	$insertRows = array();
	foreach ($data as $dataRow)
	{
		$dataRow2 = array_combine($indexed_columns, $dataRow);

		$insertData = '(';
		foreach ($columns as $columnName => $type)
		{
			// Are we restricting the length?
			if (strpos($type, 'string-') !== false)
				$insertData .= "SUBSTRING('" . $dataRow2[$columnName] . "', 1, " . substr($type, 7) . "), ";
			elseif (in_array($type, array('string', 'text')))
				$insertData .= "'" . $dataRow2[$columnName] . "', ";
			else
				$insertData .= $dataRow2[$columnName] . ", ";
		}
		$insertData = substr($insertData, 0, -2) . ')';

		$insertRows[] = $insertData;
	}

	// Determine the method of insertion.
	$queryTitle = $method == 'replace' ? 'REPLACE' : ($method == 'ignore' ? 'INSERT IGNORE' : 'INSERT');

	// Do the insert.
	db_query("
		$queryTitle INTO $table (`" . implode("`, `", $indexed_columns) . "`)
		VALUES
			" . implode(",
			", $insertRows), __FILE__, __LINE__);
}

?>