<?php

/**
 * Custom fields and filters of post
 *
 * @package CustomFieldFilterPost
 * @version 1.5
 * @author davidhs
 * @copyright 2014-2017 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License http://creativecommons.org/licenses/by/3.0/
 */

if (!defined('SMF'))
	die('Hacking attempt...');

// Adds admin areas.
function cffp_admin_areas(&$admin_areas)
{
	global $txt;

	$admin_areas['config']['areas']['modsettings']['subsections']['cffp'] = array($txt['cffp']);
}

// Adds modify modifications.
function cffp_modify_modifications(&$subActions)
{
	$subActions['cffp'] = 'cffpSettingsShow';
	$subActions['cffpeditfield'] = 'cffpSettingsEditField';
	$subActions['cffpeditgroup'] = 'cffpSettingsEditGroup';
	$subActions['cffpedittemplate'] = 'cffpSettingsEditTemplate';
}

// Gets ID column free on group and creates column if it is not exists.
function cffpGetIdColumn($id_group)
{
	global $smcFunc;

	if (empty($id_group))
		return false;

	db_extend('packages');

	// Get id_column.
	$request = $smcFunc['db_query']('', '
		SELECT id_column
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group = {int:id_group}
		ORDER BY id_column ASC',
		array(
			'id_group' => $id_group,
		)
	);
	$id_column = 1;
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		if ($id_column < $row['id_column'])
			break; // Exit while.
		else
			$id_column++;
	}
	$smcFunc['db_free_result']($request);

	// Create column if it is not exists.
	$name_field = 'column_' . $id_column;
	$list_columns = $smcFunc['db_list_columns']('{db_prefix}cffp_fields');
	if (!in_array($name_field, $list_columns))
	{
		$column = array(
			'name' => $name_field,
			'type' => 'text',
			'default' => '',
		);
		$smcFunc['db_add_column']('{db_prefix}cffp_fields', $column);
	}

	return $id_column;
}

// Gets fields of group with column in moderation list.
function cffpGetModerationListFields($id_group)
{
	global $smcFunc;

	if (empty($id_group))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT name, id_column, show_moderation_list_type, show_moderation_list_width, show_moderation_list_name
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group = {int:id_group}
			AND show_moderation_list_type <> {string:no_column}
			AND active = {int:active}
		ORDER BY field_order ASC',
		array(
			'id_group' => $id_group,
			'no_column' => 'no',
			'active' => 1,
		)
	);
	$list = array();
	$multi = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		if ('one' == $row['show_moderation_list_type'])
			$list[] = $row;
		else // multi.
		{
			$column = (!empty($row['show_moderation_list_width']) ? $row['show_moderation_list_width'] : 'auto') . ',' . $row['show_moderation_list_name'];
			if (!in_array($column, $multi))
			{
				$multi[] = $column;
				$list[] = $row;
			}
		}
	}
	$smcFunc['db_free_result']($request);

	return $list;
}

// Gets fields of record with column in search result.
function cffpGetRecordListFields($id_group)
{
	global $smcFunc;

	if (empty($id_group))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT name, id_column, show_record_list_type, show_record_list_width, show_record_list_name
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group = {int:id_group}
			AND show_record_list_type <> {string:no_column}
			AND active = {int:active}
		ORDER BY field_order ASC',
		array(
			'id_group' => $id_group,
			'no_column' => 'no',
			'active' => 1,
		)
	);
	$list = array();
	$multi = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		if ('one' == $row['show_record_list_type'])
			$list[] = $row;
		else // multi.
		{
			$column = (!empty($row['show_record_list_width']) ? $row['show_record_list_width'] : 'auto') . ',' . $row['show_record_list_name'];
			if (!in_array($column, $multi))
			{
				$multi[] = $column;
				$list[] = $row;
			}
		}
	}
	$smcFunc['db_free_result']($request);

	return $list;
}

// Gets list of settings fields of a group.
function cffpGetListSettingsFields($id_group, $only_active)
{
	global $smcFunc;

	if (empty($id_group))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT
			id_field, name, field_desc, description, field_order,
			type, length, options, options_text, mask, bbc,
			show_message_index,
			not_empty_value, unique_value, subset, active
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group = {int:id_group}' . (!$only_active ? '' : '
			AND active = {int:active}') . '
		ORDER BY field_order ASC',
		array(
			'id_group' => $id_group,
			'active' => 1,
		)
	);
	$list = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$list[] = $row;
	$smcFunc['db_free_result']($request);

	return $list;
}

// Gets list of settings groups.
function cffpGetListSettingsGroups($start, $items_per_page, $sort)
{
	global $smcFunc;

	if (!is_numeric($start) || empty($items_per_page) || empty($sort))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT
			id_group, name, description, group_order, type, show_filters, active,
			template_quick_input_regex, template_text, template_message_index_text
		FROM {db_prefix}cffp_settings_groups
		ORDER BY {raw:sort}
		LIMIT {int:start}, {int:items_per_page}',
		array(
			'sort' => $sort,
			'start' => $start,
			'items_per_page' => $items_per_page,
		)
	);
	$list = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$list[] = $row;
	$smcFunc['db_free_result']($request);

	return $list;
}

// Gets size of list of settings groups.
function cffpGetListSettingsGroupsSize()
{
	global $smcFunc;

	$request = $smcFunc['db_query']('', '
		SELECT COUNT(*)
		FROM {db_prefix}cffp_settings_groups',
		array()
	);

	list ($size) = $smcFunc['db_fetch_row']($request);
	$smcFunc['db_free_result']($request);

	return $size;
}

// Gets more information of field.
function cffpGetMoreInfoField($field)
{
	global $txt, $smcFunc;

	if (empty($field))
		return false;

	$text = array();
	switch ($field['type'])
	{
		case 'text':
		case 'textarea':
		{
			if (!empty($field['length']))
				$text[] = $txt['cffp_length'] . ': ' . $field['length'];
			if ('text' == $field['type'])
				$text[] = $txt['cffp_mask'] . ': ' . $txt['cffp_mask_' . ('regex' == $smcFunc['substr']($field['mask'], 0, 5) ? 'regex' : $field['mask'])];
			if (!empty($field['bbc']))
				$text[] = $txt['cffp_allow_bbc'];
			break;
		}
		case 'select':
		case 'combobox':
		{
			$options = explode(',', $field['options']);
			$select_multiple = array_shift($options);
			array_shift($options);

			if (0 < strlen($field['options_text']))
				$options = explode(',', $field['options_text']);

			if ('combobox' == $field['type'] && !empty($field['length']))
				$text[] = $txt['cffp_length'] . ': ' . $field['length'];
			if ($select_multiple)
				$text[] = $txt['cffp_select_multiple'];
			if (!empty($options))
				$text[] = $txt['cffp_options'] . ': ' . implode(', ', $options);
			break;
		}
		case 'radio':
		case 'check':
		{
			$options = 0 < strlen($field['options_text']) ? $field['options_text'] : $field['options'];
			if (0 < strlen($options))
				$text[] = $txt['cffp_options'] . ': ' . str_replace(',', ', ', $options);
			break;
		}
	}
	if (!empty($field['not_empty_value']))
		$text[] = $txt['cffp_not_empty_value'];
	if (!empty($field['unique_value']))
		$text[] = $txt['cffp_unique_value'];
	if (cffpIsNotEmptyString($field['subset']))
		$text[] = $txt['cffp_subset'] . ': ' . $field['subset'];

	return empty($text) ? '' : implode('<br />', $text);
}

// Gets columns multi-field of moderation list
function cffpGetModerationColumnsMulti($id_group)
{
	global $smcFunc;

	if (empty($id_group))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT show_moderation_list_width, show_moderation_list_name
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group = {int:id_group}
			AND show_moderation_list_type = {string:show_moderation_list_type}
		ORDER BY show_moderation_list_name ASC',
		array(
			'id_group' => $id_group,
			'show_moderation_list_type' => 'multi',
		)
	);
	$moderation_column_multi_array = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$moderation_column_multi_array[] = $row;
	$smcFunc['db_free_result']($request);

	return $moderation_column_multi_array;
}

// Gets columns multi-field of records list
function cffpGetRecordColumnsMulti($id_group)
{
	global $smcFunc;

	if (empty($id_group))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT show_record_list_width, show_record_list_name
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group = {int:id_group}
			AND show_record_list_type = {string:show_record_list_type}
		ORDER BY show_record_list_name ASC',
		array(
			'id_group' => $id_group,
			'show_record_list_type' => 'multi',
		)
	);
	$record_column_multi_array = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$record_column_multi_array[] = $row;
	$smcFunc['db_free_result']($request);

	return $record_column_multi_array;
}

// Gets tags for hidden filters of a group of fields.
function cffpGetHideFilter($id_group)
{
	global $smcFunc;

	if (empty($id_group))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT hide_filter
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group = {int:id_group}
			AND hide_filter <> {string:empty}
		ORDER BY hide_filter ASC',
		array(
			'id_group' => $id_group,
			'empty' => '',
		)
	);
	$hide_filter_array = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$hide_filter_array[] = $row['hide_filter'];
	$smcFunc['db_free_result']($request);

	return $hide_filter_array;
}

// Gets search pages of posts.
function cffpGetSearchPagePost()
{
	global $smcFunc;

	$search_page_array = array();

	// Get search pages with one or more than one values.
	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT search_page_name, search_page_order_by
		FROM {db_prefix}cffp_settings_groups
		WHERE type IN ({array_string:type})
			AND search_page_name <> {string:empty}',
		array(
			'type' => array('fields', 'member', 'filters'),
			'empty' => '',
		)
	);
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		if (strpos($row['search_page_name'], ','))
		{
			$search_page_name_array = explode(',', $row['search_page_name']);
			$search_page_order_by_array = explode(',', $row['search_page_order_by']);
			foreach ($search_page_name_array as $k => $search_page_name)
				$search_page_array[] = $search_page_name . ',' . $search_page_order_by_array[$k];
		}
		else
			$search_page_array[] = $row['search_page_name'] . ',' . $row['search_page_order_by'];
	}
	$smcFunc['db_free_result']($request);

	// Remove duplicate search pages.
	$search_page_array = array_unique($search_page_array);
	sort($search_page_array);

	return $search_page_array;
}

// Gets subset of a group of fields.
function cffpGetSubset($id_group)
{
	global $smcFunc;

	if (empty($id_group))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT subset
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group = {int:id_group}
			AND subset <> {string:empty}
		ORDER BY subset ASC',
		array(
			'id_group' => $id_group,
			'empty' => '',
		)
	);
	$subset_array = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$subset_array[] = $row['subset'];
	$smcFunc['db_free_result']($request);

	return $subset_array;
}

// Removes a column of group.
function cffpRemoveColumn($id_group, $id_column)
{
	global $smcFunc;

	if (empty($id_group) || empty($id_column))
		return false;

	db_extend('packages');

	$request = $smcFunc['db_query']('', '
		SELECT id_column
		FROM {db_prefix}cffp_settings_fields
		WHERE id_group <> {int:id_group}
			AND id_column = {int:id_column}',
		array(
			'id_group' => $id_group,
			'id_column' => $id_column,
		)
	);
	$num_rows = $smcFunc['db_num_rows']($request);
	$smcFunc['db_free_result']($request);

	if (!$num_rows)
	{
		$name_field = 'column_' . $id_column;
		$smcFunc['db_remove_column']('{db_prefix}cffp_fields', $name_field);
	}

	return !$num_rows;
}

// Admin area settings (edit field).
function cffpSettingsEditField($return_config = false)
{
	global $txt, $context, $smcFunc;

	if ($return_config)
		return array();

	loadTemplate('CustomFieldFilterPost');

	// Sort out the context!
	$context['gid'] = isset($_GET['gid']) ? (int) $_GET['gid'] : 0;
	$context['fid'] = isset($_GET['fid']) ? (int) $_GET['fid'] : 0;
	$context['settings_title'] = $context['fid'] ? $txt['cffp_edit_field_title'] : $txt['cffp_add_field_title'];
	$start = isset($_GET['start']) ? (int) $_GET['start'] : 0;
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'group_order';
	$context['params_url'] = ';start=' . $start . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context[$context['admin_menu_name']]['current_subsection'] = 'cffp';
	$context['sub_template'] = 'cffp_settings_edit_field';

	if ($context['gid'])
	{
		$type_filter_array = array(
			'text' => array(
				array('text', 'textarea'),
				array('text', 'empty_non_empty'),
			),
			'combobox' => array(
				array('select', 'combobox', 'radio', 'check'),
				array('text', 'select', 'combobox', 'empty_non_empty'),
			),
			'boolean' => array(
				array('boolean'),
				array('select'),
			),
		);

		if ($context['fid'])
		{
			$request = $smcFunc['db_query']('', '
				SELECT
					name, id_column,
					field_desc, field_long_desc, field_search_desc, field_search_long_desc,
					description, field_order,
					type, type_filter, length, options, options_text, mask, bbc, default_value,
					text_previous, text_next,
					enclose_type, enclose_text,
					show_message_index, show_filter_operator, hide_filter, show_filter_used_options,
					show_record_list_type, show_record_list_width, show_record_list_name,
					show_moderation_list_type, show_moderation_list_width, show_moderation_list_name,
					not_empty_value, unique_value, subset, permissions, active
				FROM {db_prefix}cffp_settings_fields
				WHERE id_field = {int:id_field}
					AND id_group = {int:id_group}',
				array(
					'id_field' => $context['fid'],
					'id_group' => $context['gid'],
				)
			);
			$row = $smcFunc['db_fetch_assoc']($request);
			$smcFunc['db_free_result']($request);

			if ($row)
			{
				if ('%' == substr($row['show_record_list_width'], -1))
				{
					$show_record_list_width_unit = '%';
					$show_record_list_width_value = substr($row['show_record_list_width'], 0, -1);
				}
				elseif ('px' == substr($row['show_record_list_width'], -2))
				{
					$show_record_list_width_unit = 'px';
					$show_record_list_width_value = substr($row['show_record_list_width'], 0, -2);
				}
				else // Empty.
				{
					$show_record_list_width_unit = 'auto';
					$show_record_list_width_value = 0;
				}
				if ('multi' == $row['show_record_list_type'])
					$show_record_list_column = (!empty($row['show_record_list_width']) ? $row['show_record_list_width'] : 'auto') . ',' . $row['show_record_list_name'];
				else
					$show_record_list_column = '';

				if ('%' == substr($row['show_moderation_list_width'], -1))
				{
					$show_moderation_list_width_unit = '%';
					$show_moderation_list_width_value = substr($row['show_moderation_list_width'], 0, -1);
				}
				elseif ('px' == substr($row['show_moderation_list_width'], -2))
				{
					$show_moderation_list_width_unit = 'px';
					$show_moderation_list_width_value = substr($row['show_moderation_list_width'], 0, -2);
				}
				else // Empty.
				{
					$show_moderation_list_width_unit = 'auto';
					$show_moderation_list_width_value = 0;
				}
				if ('multi' == $row['show_moderation_list_type'])
					$show_moderation_list_column = (!empty($row['show_moderation_list_width']) ? $row['show_moderation_list_width'] : 'auto') . ',' . $row['show_moderation_list_name'];
				else
					$show_moderation_list_column = '';

				if ('textarea' == $row['type'])
					list ($rows, $cols) = explode(',', $row['options']);
				else
				{
					$rows = 3;
					$cols = 30;
				}
				if ('select' == $row['type'] || 'combobox' == $row['type'])
				{
					$options = explode(',', $row['options']);
					$select_multiple = array_shift($options);
					$select_size = array_shift($options);
					$row['options'] = implode(',', $options);
				}
				else
				{
					$select_multiple = 0;
					$select_size = 1;
				}

				$multiple = (('select' == $row['type'] || 'combobox' == $row['type']) && $select_multiple) || 'check' == $row['type'];

				$context['params'] = array(
					'id_column' => $row['id_column'],
					'name' => $row['name'],
					'field_desc' => $row['field_desc'],
					'field_long_desc' => $row['field_long_desc'],
					'field_search_desc' => $row['field_search_desc'],
					'field_search_long_desc' => $row['field_search_long_desc'],
					'description' => $row['description'],
					'field_order' => $row['field_order'],
					'type' => $row['type'],
					'type_filter_text' => in_array($row['type'], $type_filter_array['text'][0]) ? $row['type_filter'] : $type_filter_array['text'][1][0],
					'type_filter_combobox' => in_array($row['type'], $type_filter_array['combobox'][0]) ? $row['type_filter'] : $type_filter_array['combobox'][1][1],
					'type_filter_boolean' => in_array($row['type'], $type_filter_array['boolean'][0]) ? $row['type_filter'] : $type_filter_array['boolean'][1][0],
					'length' => $row['length'],
					'rows' => $rows,
					'cols' => $cols,
					'select_multiple' => $select_multiple,
					'select_size' => $select_size,
					'options_old' => 0 < strlen($row['options']) ? explode(',', $row['options']) : array(),
					'options' => 0 < strlen($row['options']) ? explode(',', $row['options']) : array('', '', ''),
					'options_text' => 0 < strlen($row['options_text']) ? explode(',', $row['options_text']) : array('', '', ''),
					'mask' => $row['mask'],
					'regex' => 'regex' == $smcFunc['substr']($row['mask'], 0, 5) ? $smcFunc['substr']($row['mask'], 5) : '',
					'bbc' => $row['bbc'] ? true : false,
					'default_value_text' => 'text' == $row['type'] ? $row['default_value'] : '',
					'default_value_textarea' => 'textarea' == $row['type'] ? $row['default_value'] : '',
					'default_value_options' => !$multiple ? $row['default_value'] : '',
					'default_value_options_multiple' => $multiple && 0 < strlen($row['default_value']) ? explode(',', $row['default_value']) : array(),
					'default_value_boolean' => 'boolean' == $row['type'] ? $row['default_value'] : '',
					'text_previous' => $smcFunc['htmlspecialchars']($row['text_previous'], ENT_QUOTES),
					'text_next' => $smcFunc['htmlspecialchars']($row['text_next'], ENT_QUOTES),
					'enclose_type' => $row['enclose_type'],
					'enclose_text' => $smcFunc['htmlspecialchars']($row['enclose_text'], ENT_QUOTES),
					'show_message_index' => $row['show_message_index'],
					'show_filter_operator' => $row['show_filter_operator'],
					'hide_filter' => $row['hide_filter'],
					'show_filter_used_options' => $row['show_filter_used_options'],
					'show_record_list_type' => $row['show_record_list_type'],
					'show_record_list_width_unit' => $show_record_list_width_unit,
					'show_record_list_width_value' => $show_record_list_width_value,
					'show_record_list_column' => $show_record_list_column,
					'show_record_list_name' => '',
					'show_moderation_list_type' => $row['show_moderation_list_type'],
					'show_moderation_list_width_unit' => $show_moderation_list_width_unit,
					'show_moderation_list_width_value' => $show_moderation_list_width_value,
					'show_moderation_list_column' => $show_moderation_list_column,
					'show_moderation_list_name' => '',
					'not_empty_value' => $row['not_empty_value'],
					'unique_value' => $row['unique_value'],
					'subset' => $row['subset'],
					'permissions' => !empty($row['permissions']) ? explode(',', $row['permissions']) : array(),
					'active' => $row['active'],
				);
			}
		}

		// Setup the default values as needed.
		if (empty($context['params']))
		{
			$context['params'] = array(
				'id_column' => 0,
				'name' => '',
				'field_desc' => '',
				'field_long_desc' => '',
				'field_search_desc' => '',
				'field_search_long_desc' => '',
				'description' => '',
				'field_order' => 0,
				'type' => 'text',
				'type_filter_text' => $type_filter_array['text'][1][0],
				'type_filter_combobox' => $type_filter_array['combobox'][1][1],
				'type_filter_boolean' => $type_filter_array['boolean'][1][0],
				'length' => 255,
				'rows' => 4,
				'cols' => 30,
				'select_multiple' => 0,
				'select_size' => 1,
				'options_old' => array(),
				'options' => array('', '', ''),
				'options_text' => array('', '', ''),
				'mask' => 'nohtml',
				'regex' => '',
				'bbc' => 0,
				'default_value_text' => '',
				'default_value_textarea' => '',
				'default_value_options' => '',
				'default_value_options_multiple' => array(),
				'default_value_boolean' => '',
				'text_previous' => '',
				'text_next' => '',
				'enclose_type' => 'html',
				'enclose_text' => '',
				'show_message_index' => 0,
				'show_filter_operator' => 0,
				'hide_filter' => '',
				'show_filter_used_options' => 0,
				'show_record_list_type' => 'no',
				'show_record_list_width_unit' => 'auto',
				'show_record_list_width_value' => 0,
				'show_record_list_column' => '',
				'show_record_list_name' => '',
				'show_moderation_list_type' => 'no',
				'show_moderation_list_width_unit' => 'auto',
				'show_moderation_list_width_value' => 0,
				'show_moderation_list_column' => '',
				'show_moderation_list_name' => '',
				'not_empty_value' => 0,
				'unique_value' => 0,
				'subset' => '',
				'permissions' => array(),
				'active' => 1,
			);
		}

		// Others data.
		$show_record_list_type_array = array('no', 'one', 'multi');
		$context['params']['show_record_list_type_array'] = array();
		foreach ($show_record_list_type_array as $show_record_list_type)
			$context['params']['show_record_list_type_array'][$show_record_list_type] = $txt['cffp_show_list_type_' . $show_record_list_type];

		$show_record_list_width_unit_array = array('auto', '%', 'px');
		$context['params']['show_record_list_width_unit_array'] = array();
		foreach ($show_record_list_width_unit_array as $show_record_list_width_unit)
			$context['params']['show_record_list_width_unit_array'][$show_record_list_width_unit] = $txt['cffp_show_list_width_unit_' . $show_record_list_width_unit];

		$context['params']['show_record_list_column_array'] = cffpGetRecordColumnsMulti($context['gid']);

		$show_moderation_list_type_array = array('no', 'one', 'multi');
		$context['params']['show_moderation_list_type_array'] = array();
		foreach ($show_moderation_list_type_array as $show_moderation_list_type)
			$context['params']['show_moderation_list_type_array'][$show_moderation_list_type] = $txt['cffp_show_list_type_' . $show_moderation_list_type];

		$show_moderation_list_width_unit_array = array('auto', '%', 'px');
		$context['params']['show_moderation_list_width_unit_array'] = array();
		foreach ($show_moderation_list_width_unit_array as $show_moderation_list_width_unit)
			$context['params']['show_moderation_list_width_unit_array'][$show_moderation_list_width_unit] = $txt['cffp_show_list_width_unit_' . $show_moderation_list_width_unit];

		$context['params']['show_moderation_list_column_array'] = cffpGetModerationColumnsMulti($context['gid']);

		$enclose_type_array = array('html', 'bbc', 'php');
		$context['params']['enclose_type_array'] = array();
		foreach ($enclose_type_array as $enclose_type)
			$context['params']['enclose_type_array'][$enclose_type] = $txt['cffp_template_type_' . $enclose_type];

		$type_array = array('text', 'textarea', 'select', 'combobox', 'radio', 'check', 'boolean');
		$context['params']['type_array'] = array();
		foreach ($type_array as $type)
			$context['params']['type_array'][$type] = $txt['cffp_fieldtype_' . $type];

		$context['params']['type_filter_array'] = $type_filter_array;

		$context['params']['filtertype_array'] = array(
			'empty_non_empty' => $txt['cffp_filtertype_empty_non_empty'],
		);

		$mask_array = array('nohtml', 'email', 'number', 'regex');
		$context['params']['mask_array'] = array();
		foreach ($mask_array as $mask)
			$context['params']['mask_array'][$mask] = $txt['cffp_mask_' . $mask];

		$context['params']['hide_filter_array'] = cffpGetHideFilter($context['gid']);

		$context['params']['subset_array'] = cffpGetSubset($context['gid']);

		$context['params']['permissions_users'] = array(
			'guest' => $txt['guest'],
			'user' => $txt['user'],
			'localmod' => $txt['cffp_local_moderator'],
			'globalmod' => $txt['cffp_global_moderator'],
			'admin' => $txt['cffp_administrator'],
		);
		$context['params']['permissions_actions'] = array(
			'view' => $txt['view'],
			'filter' => $txt['cffp_filter'],
			'sort' => $txt['cffp_sort'],
			'search' => $txt['search'],
		);

		$request = $smcFunc['db_query']('', '
			SELECT name, type, show_stats
			FROM {db_prefix}cffp_settings_groups
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);
		$context['params']['group'] = $smcFunc['db_fetch_assoc']($request);
		$smcFunc['db_free_result']($request);

		$context['params']['group']['type_txt'] = $txt['cffp_grouptype_' . $context['params']['group']['type']];
	}

	// Are we saving?
	if (isset($_POST['save']) && $context['gid'])
	{
		checkSession('get');

		// Everyone needs a name - even the (bracket) unknown...
		if ('' == trim($_POST['name']))
			fatal_lang_error('error_cffp_field_need_name');

		// Validate inputs.
		$_POST['name'] = $smcFunc['htmlspecialchars']($_POST['name'], ENT_QUOTES);
		$_POST['field_desc'] = $smcFunc['htmlspecialchars']($_POST['field_desc'], ENT_QUOTES);
		$_POST['field_long_desc'] = $smcFunc['htmlspecialchars']($_POST['field_long_desc'], ENT_QUOTES);
		$_POST['field_search_desc'] = $smcFunc['htmlspecialchars']($_POST['field_search_desc'], ENT_QUOTES);
		$_POST['field_search_long_desc'] = $smcFunc['htmlspecialchars']($_POST['field_search_long_desc'], ENT_QUOTES);
		$_POST['description'] = $smcFunc['htmlspecialchars']($_POST['description'], ENT_QUOTES);
		$_POST['subset'] = $smcFunc['htmlspecialchars']($_POST['subset'], ENT_QUOTES);
		$_POST['hide_filter'] = $smcFunc['htmlspecialchars']($_POST['hide_filter'], ENT_QUOTES);

		if (in_array($_POST['type'], $context['params']['type_filter_array']['text'][0]))
			$type_filter = $_POST['type_filter_text'];
		elseif (in_array($_POST['type'], $context['params']['type_filter_array']['combobox'][0]))
			$type_filter = $_POST['type_filter_combobox'];
		else // in_array($_POST['type'], $context['params']['type_filter_array']['boolean'][0]).
			$type_filter = $_POST['type_filter_boolean'];

		$exists_select_option = in_array($_POST['type'], array('select', 'combobox', 'radio', 'check'));
		$select_multiple = isset($_POST['select_multiple']) ? 1 : 0;
		$bbc = !$exists_select_option && isset($_POST['bbc']) ? 1 : 0;
		$select_multiple = isset($_POST['select_multiple']) ? 1 : 0;
		$show_message_index = isset($_POST['show_message_index']) ? 1 : 0;

		if ('record' == $context['params']['group']['type'] && 'no' != $_POST['show_record_list_type'])
		{
			$show_record_list_type = $_POST['show_record_list_type'];
			if ('multi' == $show_record_list_type)
			{
				if (empty($_POST['show_record_list_column']))
				{
					if ('auto' == $_POST['show_record_list_width_unit'])
						$show_record_list_width = '';
					else // %, px.
					{
						$show_record_list_width_value = (int) $_POST['show_record_list_width_value'];
						if (empty($show_record_list_width_value))
							$show_record_list_width = '';
						else
							$show_record_list_width = $show_record_list_width_value . $_POST['show_record_list_width_unit'];
					}

					$show_record_list_name = $smcFunc['htmlspecialchars']($_POST['show_record_list_name'], ENT_QUOTES);
					if (!cffpIsNotEmptyString($show_record_list_name))
						$show_record_list_type = 'one';
				}
				else
				{
					$show_record_list_width = strtok($_POST['show_record_list_column'], ',');
					if ('auto' == $show_record_list_width)
						$show_record_list_width = '';
					$show_record_list_name = strtok("\n");
				}
			}
			else // one.
			{
				if ('auto' == $_POST['show_record_list_width_unit'])
					$show_record_list_width = '';
				else // %, px.
				{
					$show_record_list_width_value = (int) $_POST['show_record_list_width_value'];
					if (empty($show_record_list_width_value))
						$show_record_list_width = '';
					else
						$show_record_list_width = $show_record_list_width_value . $_POST['show_record_list_width_unit'];
				}
				$show_record_list_name = '';
			}
		}
		else
		{
			$show_record_list_type = 'no';
			$show_record_list_width = '';
			$show_record_list_name = '';
		}

		if ('member' == $context['params']['group']['type'] && 'no' != $_POST['show_moderation_list_type'])
		{
			$show_moderation_list_type = $_POST['show_moderation_list_type'];
			if ('multi' == $show_moderation_list_type)
			{
				if (empty($_POST['show_moderation_list_column']))
				{
					if ('auto' == $_POST['show_moderation_list_width_unit'])
						$show_moderation_list_width = '';
					else // %, px.
					{
						$show_moderation_list_width_value = (int) $_POST['show_moderation_list_width_value'];
						if (empty($show_moderation_list_width_value))
							$show_moderation_list_width = '';
						else
							$show_moderation_list_width = $show_moderation_list_width_value . $_POST['show_moderation_list_width_unit'];
					}

					$show_moderation_list_name = $smcFunc['htmlspecialchars']($_POST['show_moderation_list_name'], ENT_QUOTES);
					if (!cffpIsNotEmptyString($show_moderation_list_name))
						$show_moderation_list_type = 'one';
				}
				else
				{
					$show_moderation_list_width = strtok($_POST['show_moderation_list_column'], ',');
					if ('auto' == $show_moderation_list_width)
						$show_moderation_list_width = '';
					$show_moderation_list_name = strtok("\n");
				}
			}
			else // one.
			{
				if ('auto' == $_POST['show_moderation_list_width_unit'])
					$show_moderation_list_width = '';
				else // %, px.
				{
					$show_moderation_list_width_value = (int) $_POST['show_moderation_list_width_value'];
					if (empty($show_moderation_list_width_value))
						$show_moderation_list_width = '';
					else
						$show_moderation_list_width = $show_moderation_list_width_value . $_POST['show_moderation_list_width_unit'];
				}
				$show_moderation_list_name = '';
			}
		}
		else
		{
			$show_moderation_list_type = 'no';
			$show_moderation_list_width = '';
			$show_moderation_list_name = '';
		}

		$not_empty_value = "boolean" != $_POST['type'] && isset($_POST['not_empty_value']) ? 1 : 0;
		$unique_value = "boolean" != $_POST['type'] && isset($_POST['unique_value']) ? 1 : 0;
		$active = isset($_POST['active']) ? 1 : 0;

		if ('empty_non_empty' == $type_filter && $not_empty_value)
		{
			if (in_array($_POST['type'], $context['params']['type_filter_array']['text'][0]))
				$type_filter = $context['params']['type_filter_array']['text'][1][0];
			else // in_array($_POST['type'], $context['params']['type_filter_array']['combobox'][0])
				$type_filter = $context['params']['type_filter_array']['combobox'][1][1];
		}

		$mask = 'text' == $_POST['type'] && isset($_POST['mask']) ? $_POST['mask'] : '';
		if ('regex' == $mask && isset($_POST['regex']))
			$mask .= $_POST['regex'];

		if (in_array($_POST['type'], array('text', 'textarea', 'combobox')))
			$length = isset($_POST['length']) ? (int) $_POST['length'] : 255;
		else
			$length = 0;
		$field_order = isset($_POST['field_order']) ? (int) $_POST['field_order'] : 0;
		$select_size = isset($_POST['select_size']) ? (int) $_POST['select_size'] : 1;

		$permissions = !empty($_POST['permissions']) ? implode(',', $_POST['permissions']) : '';

		$multiple = (('select' == $_POST['type'] || 'combobox' == $_POST['type']) && $select_multiple) || 'check' == $_POST['type'];
		$options = '';
		$options_text = '';
		$option_new_array = array();
		$default = '';
		if ($exists_select_option && !empty($_POST['select_option']))
		{
			$option_array = array();
			$option_text_array = array();
			$default_array = array();
			foreach ($_POST['select_option'] as $k => $v)
			{
				// Clean, clean, clean...
				$v = $smcFunc['htmlspecialchars']($v, ENT_QUOTES);
				$v = strtr($v, array(',' => ''));
				$v = trim($v);

				if (!cffpIsNotEmptyString($v))
					continue;

				if ('combobox' == $_POST['type'] && $_POST['length'])
					$v = $smcFunc['substr']($v, 0, $_POST['length']);

				if (in_array($v, $option_array))
					continue;

				$option_array[] = $v;
				$option_new_array[$k] = $v;
				if ('combobox' != $_POST['type'] && isset($_POST['select_option_text'][$k]))
				{
					$t = $smcFunc['htmlspecialchars']($_POST['select_option_text'][$k], ENT_QUOTES);
					$t = strtr($t, array(',' => ''));
					$t = trim($t);
					$option_text_array[] = cffpIsNotEmptyString($t) ? $t : $v;
				}

				// Is it default?
				if (!$multiple && isset($_POST['default_value_options']) && $_POST['default_value_options'] == $k)
					$default = $v;
				elseif ($multiple && isset($_POST['default_value_options_multiple']) && in_array($k, $_POST['default_value_options_multiple']))
					$default_array[] = $v;
			}
			$options = implode(',', $option_array);
			if ($multiple)
				$default = implode(',', $default_array);
			$options_text = implode(',', $option_text_array);
			if ($options_text == $options)
				$options_text = '';

			if ('combobox' == $type_filter && 0 < strlen($options_text))
				$type_filter = 'select';
		}
		elseif ('text' == $_POST['type'])
		{
			$default = $smcFunc['htmlspecialchars']($_POST['default_value_text'], ENT_QUOTES);
			if ($_POST['length'])
				$default = $smcFunc['substr']($default, 0, $_POST['length']);
			if (cffpIsNotEmptyString($default))
			{
				$error = cffpValidateMask($mask, $default);
				if (!empty($error))
					$default = '';
			}
		}
		elseif ('textarea' == $_POST['type'])
		{
			$options = (int) $_POST['rows'] . ',' . (int) $_POST['cols'];
			$default = $smcFunc['htmlspecialchars']($_POST['default_value_textarea'], ENT_QUOTES);
			if ($_POST['length'])
				$default = $smcFunc['substr']($default, 0, $_POST['length']);
		}
		elseif ('boolean' == $_POST['type'])
			$default = isset($_POST['default_value_boolean']) ? '1' : '0';

		$show_filter_operator = isset($_POST['show_filter_operator']) && 'boolean' != $_POST['type'] && 'empty_non_empty' != $type_filter ? 1 : 0;
		$show_filter_used_options = isset($_POST['show_filter_used_options']) && in_array($type_filter, array('select', 'combobox')) ? 1 : 0;

		if (in_array($_POST['type'], array('select', 'radio', 'check')) && 0 == strlen($options))
			fatal_lang_error('error_cffp_field_need_options');
		elseif (in_array($_POST['type'], array('select', 'combobox')))
		{
			$options2 = $select_multiple . ',' . $select_size;
			if (0 < strlen($options))
				$options2 .= ',' . $options;
		}
		else
			$options2 = $options;

		// Do the insertion/updates.
		if ($context['fid'])
		{
			$smcFunc['db_query']('', '
				UPDATE {db_prefix}cffp_settings_fields
				SET
					name = {string:name},
					field_desc = {string:field_desc}, field_long_desc = {string:field_long_desc},
					field_search_desc = {string:field_search_desc}, field_search_long_desc = {string:field_search_long_desc},
					description = {string:description}, field_order = {int:field_order},
					type = {string:type}, type_filter = {string:type_filter},
					length = {int:length},
					options = {string:options}, options_text = {string:options_text},
					mask = {string:mask}, bbc = {int:bbc},
					default_value = {string:default_value},
					text_previous = {string:text_previous}, text_next = {string:text_next},
					enclose_type = {string:enclose_type}, enclose_text = {string:enclose_text},
					show_message_index = {int:show_message_index},
					show_filter_operator = {int:show_filter_operator}, hide_filter = {string:hide_filter},
					show_filter_used_options = {int:show_filter_used_options},
					show_record_list_type = {string:show_record_list_type}, show_record_list_width = {string:show_record_list_width},
					show_record_list_name = {string:show_record_list_name},
					show_moderation_list_type = {string:show_moderation_list_type}, show_moderation_list_width = {string:show_moderation_list_width},
					show_moderation_list_name = {string:show_moderation_list_name},
					not_empty_value = {int:not_empty_value}, unique_value = {int:unique_value},
					subset = {string:subset},
					permissions = {string:permissions}, active = {int:active}
				WHERE id_field = {int:id_field}
					AND id_group = {int:id_group}',
				array(
					'id_field' => $context['fid'],
					'id_group' => $context['gid'],
					'name' => $_POST['name'],
					'field_desc' => $_POST['field_desc'],
					'field_long_desc' => $_POST['field_long_desc'],
					'field_search_desc' => $_POST['field_search_desc'],
					'field_search_long_desc' => $_POST['field_search_long_desc'],
					'description' => $_POST['description'],
					'field_order' => $field_order,
					'type' => $_POST['type'],
					'type_filter' => $type_filter,
					'length' => $length,
					'options' => $options2,
					'options_text' => $options_text,
					'mask' => $mask,
					'bbc' => $bbc,
					'default_value' => $default,
					'text_previous' => $_POST['text_previous'],
					'text_next' => $_POST['text_next'],
					'enclose_type' => $_POST['enclose_type'],
					'enclose_text' => $_POST['enclose_text'],
					'show_message_index' => $show_message_index,
					'show_filter_operator' => $show_filter_operator,
					'hide_filter' => $_POST['hide_filter'],
					'show_filter_used_options' => $show_filter_used_options,
					'show_record_list_type' => $show_record_list_type,
					'show_record_list_width' => $show_record_list_width,
					'show_record_list_name' => $show_record_list_name,
					'show_moderation_list_type' => $show_moderation_list_type,
					'show_moderation_list_width' => $show_moderation_list_width,
					'show_moderation_list_name' => $show_moderation_list_name,
					'not_empty_value' => $not_empty_value,
					'unique_value' => $unique_value,
					'subset' => $_POST['subset'],
					'permissions' => $permissions,
					'active' => $active,
				)
			);

			// Fix old values with new settings.
			$name_field = 'column_' . $context['params']['id_column'];
			$exists_select_option_old = in_array($context['params']['type'], array('select', 'combobox', 'radio', 'check'));
			if ('boolean' == $_POST['type'] && 'boolean' != $context['params']['type'])
			{
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}cffp_fields
					SET {raw:name_field} = {string:new_value}
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
						'name_field' => $name_field,
						'new_value' => '0',
					)
				);
			}
			elseif ($exists_select_option && !$exists_select_option_old)
			{
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}cffp_fields
					SET {raw:name_field} = {string:new_value}
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
						'name_field' => $name_field,
						'new_value' => '',
					)
				);
			}
			elseif ($exists_select_option && $exists_select_option_old && 'combobox' != $_POST['type'] && !empty($context['params']['options_old']))
			{
				if (empty($option_new_array))
				{
					$smcFunc['db_query']('', '
						UPDATE {db_prefix}cffp_fields
						SET {raw:name_field} = {string:new_value}
						WHERE id_group = {int:id_group}',
						array(
							'id_group' => $context['gid'],
							'name_field' => $name_field,
							'new_value' => '',
						)
					);
				}
				else
				{
					$option_delete_array = array_diff($context['params']['options_old'], $option_new_array);
					if (!empty($option_delete_array))
					{
						$smcFunc['db_query']('', '
							UPDATE {db_prefix}cffp_fields
							SET {raw:name_field} = {string:new_value}
							WHERE id_group = {int:id_group}
								AND {raw:name_field} IN ({array_string:old_values})',
							array(
								'id_group' => $context['gid'],
								'name_field' => $name_field,
								'new_value' => '',
								'old_values' => $option_delete_array,
							)
						);

						foreach ($option_delete_array as $option)
						{
							$smcFunc['db_query']('', '
								UPDATE {db_prefix}cffp_fields
								SET {raw:name_field} = REPLACE({raw:name_field}, {string:old_value}, {string:new_value})
								WHERE id_group = {int:id_group}
									AND {raw:name_field} LIKE {string:like_old_value}',
								array(
									'id_group' => $context['gid'],
									'name_field' => $name_field,
									'new_value' => ',',
									'old_value' => ',' . $option . ',',
									'like_old_value' => '%,' . $option . ',%',
								)
							);
							$smcFunc['db_query']('', '
								UPDATE {db_prefix}cffp_fields
								SET {raw:name_field} = REPLACE({raw:name_field}, {string:old_value}, {string:new_value})
								WHERE id_group = {int:id_group}
									AND {raw:name_field} LIKE {string:like_old_value}',
								array(
									'id_group' => $context['gid'],
									'name_field' => $name_field,
									'new_value' => '',
									'old_value' => $option . ',',
									'like_old_value' => $option . ',%',
								)
							);
							$smcFunc['db_query']('', '
								UPDATE {db_prefix}cffp_fields
								SET {raw:name_field} = REPLACE({raw:name_field}, {string:old_value}, {string:new_value})
								WHERE id_group = {int:id_group}
									AND {raw:name_field} LIKE {string:like_old_value}',
								array(
									'id_group' => $context['gid'],
									'name_field' => $name_field,
									'new_value' => '',
									'old_value' => ',' . $option,
									'like_old_value' => '%,' . $option,
								)
							);
						}
					}
				}
			}

			if ($not_empty_value && !$context['params']['not_empty_value'])
			{
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}cffp_fields
					SET {raw:name_field} = {string:new_value}
					WHERE id_group = {int:id_group}
						AND {raw:name_field} = {string:value}',
					array(
						'id_group' => $context['gid'],
						'name_field' => $name_field,
						'value' => '',
						'new_value' => '0',
					)
				);
			}

			if ($unique_value && !$context['params']['unique_value'])
			{
				$request = $smcFunc['db_query']('', '
					SELECT COUNT(*) AS num, {raw:name_field}
					FROM {db_prefix}cffp_fields
					WHERE id_group = {int:id_group}
						AND {raw:name_field} <> {string:empty_value}
					GROUP BY {raw:name_field}
					HAVING num > {int:unique}',
					array(
						'id_group' => $context['gid'],
						'name_field' => $name_field,
						'empty_value' => '',
						'unique' => 1,
					)
				);
				while ($row = $smcFunc['db_fetch_assoc']($request))
				{
					$smcFunc['db_query']('', '
						UPDATE {db_prefix}cffp_fields
						SET {raw:name_field} = {string:new_value}
						WHERE id_group = {int:id_group}
							AND {raw:name_field} = {string:value}',
						array(
							'id_group' => $context['gid'],
							'name_field' => $name_field,
							'value' => $row[$name_field],
							'new_value' => '',
						)
					);
				}
				$smcFunc['db_free_result']($request);
			}

			//TO DO: Adjust based on new text length limits.

		}
		else
		{
			$id_column = cffpGetIdColumn($context['gid']);
			$smcFunc['db_insert']('',
				'{db_prefix}cffp_settings_fields',
				array(
					'id_group' => 'int', 'id_column' => 'int',
					'name' => 'string',
					'field_desc' => 'string', 'field_long_desc' => 'string',
					'field_search_desc' => 'string', 'field_search_long_desc' => 'string',
					'description' => 'string', 'field_order' => 'int',
					'type' => 'string', 'type_filter' => 'string',
					'length' => 'int',
					'options' => 'string', 'options_text' => 'string',
					'mask' => 'string', 'bbc' => 'int',
					'default_value' => 'string',
					'text_previous' => 'string', 'text_next' => 'string',
					'enclose_type' => 'string', 'enclose_text' => 'string',
					'show_message_index' => 'int',
					'show_filter_operator' => 'int', 'hide_filter' => 'string',
					'show_filter_used_options' => 'int',
					'show_record_list_type' => 'string', 'show_record_list_width' => 'string',
					'show_record_list_name' => 'string',
					'show_moderation_list_type' => 'string', 'show_moderation_list_width' => 'string',
					'show_moderation_list_name' => 'string',
					'not_empty_value' => 'int', 'unique_value' => 'int',
					'subset' => 'string',
					'permissions' => 'string', 'active' => 'int',
				),
				array(
					$context['gid'], $id_column,
					$_POST['name'],
					$_POST['field_desc'], $_POST['field_long_desc'],
					$_POST['field_search_desc'], $_POST['field_search_long_desc'],
					$_POST['description'], $field_order,
					$_POST['type'], $type_filter,
					$length,
					$options2, $options_text,
					$mask, $bbc,
					$default,
					$_POST['text_previous'], $_POST['text_next'],
					$_POST['enclose_type'], $_POST['enclose_text'],
					$show_message_index,
					$show_filter_operator, $_POST['hide_filter'],
					$show_filter_used_options,
					$show_record_list_type, $show_record_list_width,
					$show_record_list_name,
					$show_moderation_list_type, $show_moderation_list_width,
					$show_moderation_list_name,
					$not_empty_value, $unique_value,
					$_POST['subset'],
					$permissions, $active,
				),
				array('id_field')
			);
		}

		if ($context['params']['group']['show_stats'])
			cffpUpdateStats($context['gid'], 0, true);
	}
	// Deleting?
	elseif (isset($_GET['delete']) && $context['gid'] && $context['fid'])
	{
		checkSession('get');

		// Delete field on posts.
		$request = $smcFunc['db_query']('', '
			SELECT id_column
			FROM {db_prefix}cffp_settings_fields
			WHERE id_field = {int:id_field}',
			array(
				'id_field' => $context['fid'],
			)
		);
		$row = $smcFunc['db_fetch_assoc']($request);
		$smcFunc['db_free_result']($request);

		if ($row)
		{
			if (!cffpRemoveColumn($context['gid'], $row['id_column']))
			{
				$name_field = 'column_' . $row['id_column'];
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}cffp_fields
					SET {raw:name_field} = {string:new_value}
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
						'name_field' => $name_field,
						'new_value' => '',
					)
				);
			}
		}

		// Finally delete setting field.
		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}cffp_settings_fields
			WHERE id_field = {int:id_field}',
			array(
				'id_field' => $context['fid'],
			)
		);

		if ($context['params']['group']['show_stats'])
			cffpUpdateStats($context['gid'], 0, true);
	}

	if (isset($_GET['delete']) || isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=admin;area=modsettings;sa=cffp' . $context['params_url']);
	}
}

// Admin area settings (edit group).
function cffpSettingsEditGroup($return_config = false)
{
	global $txt, $context, $smcFunc, $sourcedir;

	if ($return_config)
		return array();

	loadTemplate('CustomFieldFilterPost');

	// Sort out the context!
	$context['gid'] = isset($_GET['gid']) ? (int) $_GET['gid'] : 0;
	$context['settings_title'] = $context['gid'] ? $txt['cffp_edit_group_title'] : $txt['cffp_add_group_title'];
	$start = isset($_GET['start']) ? (int) $_GET['start'] : 0;
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'group_order';
	$context['params_url'] = ';start=' . $start . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context[$context['admin_menu_name']]['current_subsection'] = 'cffp';
	$context['sub_template'] = 'cffp_settings_edit_group';

	if ($context['gid'])
	{
		$request = $smcFunc['db_query']('', '
			SELECT
				id_group, name, description, group_order, text_input_before, text_input, text_output,
				boards, child_boards, type,
				show_group_name_message_index, show_group_name_filter, show_group_name_search,
				show_fields_name, show_fields_set,
				show_empty_fields_post, show_empty_fields_message_index,
				show_poster_data, show_modified_data,
				show_actions_as,
				show_member_expanded, show_member_expanded_data,
				show_member_expanded_data_recent, show_member_expanded_new,
				show_member_recent_at_top, member_max_height, member_one_value,
				show_stats,
				show_stats_post_text,
				show_stats_post_chart_average, show_stats_post_chart_hbars,
				show_stats_post_chart_vbars, show_stats_post_summary,
				show_stats_message_index_text,
				show_stats_message_index_chart_average, show_stats_message_index_chart_hbars,
				show_stats_message_index_chart_vbars, show_stats_message_index_summary,
				show_stats_order_by,
				show_sticky, show_locked,
				show_first_post_topic, show_replies_topic, no_show_with_subject_prefix,
				show_in_topic, show_filters, show_filters_operator, javascript,
				search_page_name, search_page_order_by,
				counter_show, counter_name,
				top_poster_show, top_poster_name,
				profile_list_show, profile_list_name,
				moderation_list_show, moderation_list_menu, moderation_list_name, moderation_list_order_by,
				permissions, active
			FROM {db_prefix}cffp_settings_groups
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);
		$row = $smcFunc['db_fetch_assoc']($request);
		$smcFunc['db_free_result']($request);

		if ($row)
		{
			$context['params'] = array(
				'name' => $row['name'],
				'description' => $row['description'],
				'group_order' => $row['group_order'],
				'text_input_before' => $row['text_input_before'],
				'text_input' => $row['text_input'],
				'text_output' => $row['text_output'],
				'boards' => explode(',', $row['boards']),
				'child_boards' => $row['child_boards'],
				'type' => $row['type'],
				'show_group_name_message_index' => $row['show_group_name_message_index'],
				'show_group_name_filter' => $row['show_group_name_filter'],
				'show_group_name_search' => $row['show_group_name_search'],
				'show_fields_name' => $row['show_fields_name'],
				'show_fields_set' => $row['show_fields_set'],
				'show_empty_fields_post' => $row['show_empty_fields_post'],
				'show_empty_fields_message_index' => $row['show_empty_fields_message_index'],
				'show_poster_data' => $row['show_poster_data'],
				'show_modified_data' => $row['show_modified_data'],
				'show_actions_as' => $row['show_actions_as'],
				'show_member_expanded' => $row['show_member_expanded'],
				'show_member_expanded_data' => $row['show_member_expanded_data'],
				'show_member_expanded_data_recent' => $row['show_member_expanded_data_recent'],
				'show_member_expanded_new' => $row['show_member_expanded_new'],
				'show_member_recent_at_top' => $row['show_member_recent_at_top'],
				'member_max_height' => $row['member_max_height'],
				'member_one_value' => $row['member_one_value'],
				'show_stats' => $row['show_stats'],
				'show_stats_post_text' => $row['show_stats_post_text'],
				'show_stats_post_chart_average' => !empty($row['show_stats_post_chart_average']) ? explode(',', $row['show_stats_post_chart_average']) : array(),
				'show_stats_post_chart_hbars' => !empty($row['show_stats_post_chart_hbars']) ? explode(',', $row['show_stats_post_chart_hbars']) : array(),
				'show_stats_post_chart_vbars' => !empty($row['show_stats_post_chart_vbars']) ? explode(',', $row['show_stats_post_chart_vbars']) : array(),
				'show_stats_post_summary' => !empty($row['show_stats_post_summary']) ? explode(',', $row['show_stats_post_summary']) : array(),
				'show_stats_message_index_text' => $row['show_stats_message_index_text'],
				'show_stats_message_index_chart_average' => !empty($row['show_stats_message_index_chart_average']) ? explode(',', $row['show_stats_message_index_chart_average']) : array(),
				'show_stats_message_index_chart_hbars' => !empty($row['show_stats_message_index_chart_hbars']) ? explode(',', $row['show_stats_message_index_chart_hbars']) : array(),
				'show_stats_message_index_chart_vbars' => !empty($row['show_stats_message_index_chart_vbars']) ? explode(',', $row['show_stats_message_index_chart_vbars']) : array(),
				'show_stats_message_index_summary' => !empty($row['show_stats_message_index_summary']) ? explode(',', $row['show_stats_message_index_summary']) : array(),
				'show_stats_order_by' => !empty($row['show_stats_order_by']) ? explode(',', $row['show_stats_order_by']) : array(),
				'show_sticky' => $row['show_sticky'],
				'show_locked' => $row['show_locked'],
				'show_first_post_topic' => $row['show_first_post_topic'],
				'show_replies_topic' => $row['show_replies_topic'],
				'no_show_with_subject_prefix' => $row['no_show_with_subject_prefix'],
				'show_in_topic' => $row['show_in_topic'],
				'show_filters' => !empty($row['show_filters']) ? explode(',', $row['show_filters']) : array(),
				'show_filters_operator' => !empty($row['show_filters_operator']) ? explode(',', $row['show_filters_operator']) : array(),
				'javascript' => $row['javascript'],
				'search_page_post_default' => 'record' != $row['type'] ? !cffpIsNotEmptyString($row['search_page_name']) : 1,
				'search_page_name_post' => 'record' != $row['type'] && cffpIsNotEmptyString($row['search_page_name']) ? explode(',', $row['search_page_name']) : array('', '', ''),
				'search_page_name_record' => 'record' == $row['type'] ? $row['search_page_name'] : '',
				'search_page_order_by_post' => 'record' != $row['type'] && cffpIsNotEmptyString($row['search_page_name']) ? explode(',', $row['search_page_order_by']) : array('m.poster_time desc', 'm.poster_time desc', 'm.poster_time desc'),
				'search_page_order_by_record' => 'record' == $row['type'] ? $row['search_page_order_by'] : 'cffpf0.poster_time desc',
				'counter_show' => $row['counter_show'],
				'counter_name' => $row['counter_name'],
				'top_poster_show' => $row['top_poster_show'],
				'top_poster_name' => $row['top_poster_name'],
				'profile_list_show' => $row['profile_list_show'],
				'profile_list_name' => $row['profile_list_name'],
				'moderation_list_show' => $row['moderation_list_show'],
				'moderation_list_menu' => $row['moderation_list_menu'],
				'moderation_list_name' => $row['moderation_list_name'],
				'moderation_list_order_by' => $row['moderation_list_order_by'],
				'permissions' => !empty($row['permissions']) ? explode(',', $row['permissions']) : array(),
				'active' => $row['active'],
			);
		}
	}

	// Setup the default values as needed.
	if (empty($context['params']))
	{
		$context['params'] = array(
			'name' => '',
			'description' => '',
			'group_order' => 0,
			'text_input_before' => '',
			'text_input' => '',
			'text_output' => '',
			'boards' => array(),
			'child_boards' => 0,
			'type' => 'fields',
			'show_group_name_message_index' => 0,
			'show_group_name_filter' => 0,
			'show_group_name_search' => 0,
			'show_fields_name' => 1,
			'show_fields_set' => 0,
			'show_empty_fields_post' => 0,
			'show_empty_fields_message_index' => 0,
			'show_poster_data' => 1,
			'show_modified_data' => 1,
			'show_actions_as' => 'icon_text',
			'show_member_expanded' => 0,
			'show_member_expanded_data' => 0,
			'show_member_expanded_data_recent' => 0,
			'show_member_expanded_new' => 0,
			'show_member_recent_at_top' => 0,
			'member_max_height' => 0,
			'member_one_value' => 0,
			'show_stats' => 0,
			'show_stats_post_text' => '',
			'show_stats_post_chart_average' => array(),
			'show_stats_post_chart_hbars' => array(),
			'show_stats_post_chart_vbars' => array(),
			'show_stats_post_summary' => array(),
			'show_stats_message_index_text' => '',
			'show_stats_message_index_chart_average' => array(),
			'show_stats_message_index_chart_hbars' => array(),
			'show_stats_message_index_chart_vbars' => array(),
			'show_stats_message_index_summary' => array(),
			'show_stats_order_by' => array(),
			'show_sticky' => null,
			'show_locked' => null,
			'show_first_post_topic' => 1,
			'show_replies_topic' => 1,
			'no_show_with_subject_prefix' => '',
			'show_in_topic' => 'no',
			'show_filters' => array(),
			'show_filters_operator' => array(),
			'javascript' => '',
			'search_page_post_default' => 1,
			'search_page_name_post' => array('', '', ''),
			'search_page_name_record' => '',
			'search_page_order_by_post' => array('m.poster_time desc', 'm.poster_time desc', 'm.poster_time desc'),
			'search_page_order_by_record' => 'cffpf0.poster_time desc',
			'counter_show' => 0,
			'counter_name' => '',
			'top_poster_show' => 0,
			'top_poster_name' => '',
			'profile_list_show' => 0,
			'profile_list_name' => '',
			'moderation_list_show' => 0,
			'moderation_list_menu' => 0,
			'moderation_list_name' => '',
			'moderation_list_order_by' => '',
			'permissions' => array(),
			'active' => 1,
		);
	}

	// Others data.
	$type_array = array('fields', 'record', 'member', 'filters');
	$context['params']['type_array'] = array();
	foreach ($type_array as $type)
		$context['params']['type_array'][$type] = $txt['cffp_grouptype_' . $type];

	$show_actions_as_array = array('button', 'icon_text', 'icon', 'text');
	$context['params']['show_actions_as_array'] = array();
	foreach ($show_actions_as_array as $show_actions_as)
		$context['params']['show_actions_as_array'][$show_actions_as] = $txt['cffp_show_actions_as_' . $show_actions_as];

	$show_in_topic_array = array('no', 'begin_all', 'end_all', 'begin_first', 'end_first');
	$context['params']['show_in_topic_array'] = array();
	foreach ($show_in_topic_array as $show_in_topic)
		$context['params']['show_in_topic_array'][$show_in_topic] = $txt['cffp_show_in_topic_' . $show_in_topic];

	$context['params']['stats_position'] = array('post', 'message_index');

	$show_stats_summary_array = array('average' => true, 'sum' => true, 'max' => true, 'min' => true, 'highest' => true, 'lowest' => true, 'most_frequent' => false, 'least_frequent' => false, 'total' => false);
	$context['params']['show_stats_summary_array'] = array();
	foreach ($show_stats_summary_array as $show_stats_summary => $is_numeric)
		$context['params']['show_stats_summary_array'][$show_stats_summary] = array($txt['cffp_show_stats_summary_' . $show_stats_summary], $is_numeric);

	$context['params']['show_stats_summary2_array'] = $context['params']['show_stats_summary_array'];
	unset($context['params']['show_stats_summary2_array']['max']);
	unset($context['params']['show_stats_summary2_array']['min']);

	$context['params']['show_filters_array'] = cffpGetSettingsPredefinedFilters();

	$context['params']['search_page_order_by_post_array'] = array(
		'm.subject asc' => $txt['subject'] . ' (' . $txt['cffp_order_by_asc'] . ')',
		'm.subject desc' => $txt['subject'] . ' (' . $txt['cffp_order_by_desc'] . ')',
		'm.poster_time asc' => $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_asc'] . ')',
		'm.poster_time desc' => $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_desc'] . ')',
		'display_name asc' => $txt['posted_by'] . ' (' . $txt['cffp_order_by_asc'] . ')',
		'display_name desc' => $txt['posted_by'] . ' (' . $txt['cffp_order_by_desc'] . ')',
	);

	$fields = $context['gid'] ? cffpGetRecordListFields($context['gid']) : array();
	$context['params']['search_page_order_by_record_array'] = array();
	foreach ($fields as $field)
	{
		$context['params']['search_page_order_by_record_array']['cffpf0.column_' . $field['id_column'] . ' asc'] = $field['name'] . ' (' . $txt['cffp_order_by_asc'] . ')';
		$context['params']['search_page_order_by_record_array']['cffpf0.column_' . $field['id_column'] . ' desc'] = $field['name'] . ' (' . $txt['cffp_order_by_desc'] . ')';
	}
	$context['params']['search_page_order_by_record_array']['cffpf0.poster_time asc'] = $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_asc'] . ')';
	$context['params']['search_page_order_by_record_array']['cffpf0.poster_time desc'] = $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_desc'] . ')';
	$context['params']['search_page_order_by_record_array']['display_name asc'] = $txt['posted_by'] . ' (' . $txt['cffp_order_by_asc'] . ')';
	$context['params']['search_page_order_by_record_array']['display_name desc'] = $txt['posted_by'] . ' (' . $txt['cffp_order_by_desc'] . ')';

	foreach ($context['params']['search_page_order_by_post'] as $k => $v)
	{
		if (!isset($context['params']['search_page_order_by_post_array'][$v]))
			$context['params']['search_page_order_by_post'][$k] = 'm.poster_time desc';
	}

	if (!isset($context['params']['search_page_order_by_record_array'][$context['params']['search_page_order_by_record']]))
		$context['params']['search_page_order_by_record'] = 'cffpf0.poster_time desc';

	$context['params']['search_page_post_more_array'] = cffpGetSearchPagePost();

	$context['params']['moderation_list_order_by_array'] = array(
		'm.subject asc' => $txt['subject'] . ' (' . $txt['cffp_order_by_asc'] . ')',
		'm.subject desc' => $txt['subject'] . ' (' . $txt['cffp_order_by_desc'] . ')',
	);
	$fields = $context['gid'] ? cffpGetModerationListFields($context['gid']) : array();
	foreach ($fields as $field)
	{
		$context['params']['moderation_list_order_by_array']['cffpf.column_' . $field['id_column'] . ' asc'] = $field['name'] . ' (' . $txt['cffp_order_by_asc'] . ')';
		$context['params']['moderation_list_order_by_array']['cffpf.column_' . $field['id_column'] . ' desc'] = $field['name'] . ' (' . $txt['cffp_order_by_desc'] . ')';
	}
	$context['params']['moderation_list_order_by_array']['cffpf.poster_time asc'] = $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_asc'] . ')';
	$context['params']['moderation_list_order_by_array']['cffpf.poster_time desc'] = $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_desc'] . ')';
	$context['params']['moderation_list_order_by_array']['display_name asc'] = $txt['posted_by'] . ' (' . $txt['cffp_order_by_asc'] . ')';
	$context['params']['moderation_list_order_by_array']['display_name desc'] = $txt['posted_by'] . ' (' . $txt['cffp_order_by_desc'] . ')';

	if (!isset($context['params']['moderation_list_order_by_array'][$context['params']['moderation_list_order_by']]))
		$context['params']['moderation_list_order_by'] = 'cffpf.poster_time desc';

	$context['params']['permissions_users'] = array(
		'guest' => $txt['guest'],
		'user' => $txt['user'],
		'owner' => array($txt['cffp_owner_post'], $txt['cffp_owner_group']),
		'localmod' => $txt['cffp_local_moderator'],
		'globalmod' => $txt['cffp_global_moderator'],
		'admin' => $txt['cffp_administrator'],
	);
	$context['params']['permissions_actions'] = array(
		'view' => array($txt['view'], $txt['cffp_view_message_index']),
		'stats' => $txt['cffp_view_stats'],
		'add' => $txt['cffp_add'],
		'modify' => $txt['modify'],
		'remove' => $txt['remove'],
		'move' => $txt['cffp_move'],
		'search' => $txt['search'],
	);

	require_once ($sourcedir . '/Subs-MessageIndex.php');
	$boardListOptions = array(
		'not_redirection' => true,
	);
	$context['params']['all_boards'] = getBoardList($boardListOptions);

	// Are we saving?
	if (isset($_POST['save']))
	{
		checkSession('get');

		// Everyone needs a name - even the (bracket) unknown...
		if ('' == trim($_POST['name']))
			fatal_lang_error('error_cffp_group_need_name');

		// Validate inputs.
		$_POST['name'] = $smcFunc['htmlspecialchars']($_POST['name'], ENT_QUOTES);
		$_POST['description'] = $smcFunc['htmlspecialchars']($_POST['description'], ENT_QUOTES);
		$_POST['text_input'] = 'filters' != $_POST['type'] ? $smcFunc['htmlspecialchars']($_POST['text_input'], ENT_QUOTES) : '';
		$_POST['text_output'] = 'filters' != $_POST['type'] ? $smcFunc['htmlspecialchars']($_POST['text_output'], ENT_QUOTES) : '';

		$show_group_name_message_index = 'filters' != $_POST['type'] && isset($_POST['show_group_name_message_index']) ? 1 : 0;
		$show_fields_name = 'filters' != $_POST['type'] && isset($_POST['show_fields_name']) ? 1 : 0;
		$show_empty_fields_post = 'filters' != $_POST['type'] && isset($_POST['show_empty_fields_post']) ? 1 : 0;
		$show_empty_fields_message_index = 'filters' != $_POST['type'] && isset($_POST['show_empty_fields_message_index']) ? 1 : 0;
		$show_actions_as = 'filters' != $_POST['type'] ? $_POST['show_actions_as'] : 'icon_text';

		$show_fields_set = 'fields' == $_POST['type'] && isset($_POST['show_fields_set']) ? 1 : 0;

		$_POST['text_input_before'] = 'record' == $_POST['type'] ? $smcFunc['htmlspecialchars']($_POST['text_input_before'], ENT_QUOTES) : '';
		$show_poster_data = 'record' == $_POST['type'] && isset($_POST['show_poster_data']) ? 1 : 0;
		$show_modified_data = $show_poster_data && isset($_POST['show_modified_data']) ? 1 : 0;
		$counter_show = 'record' == $_POST['type'] && isset($_POST['counter_show']) ? 1 : 0;
		$counter_name = $counter_show ? $_POST['counter_name'] : '';
		$top_poster_show = 'record' == $_POST['type'] && isset($_POST['top_poster_show']) ? 1 : 0;
		$top_poster_name = $top_poster_show ? $_POST['top_poster_name'] : '';
		$profile_list_show = 'record' == $_POST['type'] && isset($_POST['profile_list_show']) ? 1 : 0;
		$profile_list_name = $profile_list_show ? $_POST['profile_list_name'] : '';

		$show_member_expanded = 'member' == $_POST['type'] ? $_POST['show_member_expanded'] : 0;
		$show_member_expanded_data = 'member' == $_POST['type'] ? $_POST['show_member_expanded_data'] : 0;
		$show_member_expanded_data_recent = 'member' == $_POST['type'] && !empty($_POST['show_member_expanded_data_recent']) ? (int) $_POST['show_member_expanded_data_recent'] : 0;
		$show_member_expanded_new = 'member' == $_POST['type'] ? $_POST['show_member_expanded_new'] : 0;
		$show_member_recent_at_top = 'member' == $_POST['type'] && isset($_POST['show_member_recent_at_top']) ? 1 : 0;
		$member_max_height = 'member' == $_POST['type'] && !empty($_POST['member_max_height']) ? (int) $_POST['member_max_height'] : 0;
		$member_one_value = 'member' == $_POST['type'] && isset($_POST['member_one_value']) ? 1 : 0;
		$show_stats = 'member' == $_POST['type'] && isset($_POST['show_stats']) ? 1 : 0;
		$show_stats_post_text = $show_stats ? $smcFunc['htmlspecialchars']($_POST['show_stats_post_text'], ENT_QUOTES) : '';
		$show_stats_post_chart_average = $show_stats && !empty($_POST['show_stats_post_chart_average']) && in_array('show', $_POST['show_stats_post_chart_average']) ? implode(',', $_POST['show_stats_post_chart_average']) : '';
		$show_stats_post_chart_hbars = $show_stats && !empty($_POST['show_stats_post_chart_hbars']) && in_array('show', $_POST['show_stats_post_chart_hbars']) ? implode(',', $_POST['show_stats_post_chart_hbars']) : '';
		$show_stats_post_chart_vbars = $show_stats && !empty($_POST['show_stats_post_chart_vbars']) && in_array('show', $_POST['show_stats_post_chart_vbars']) ? implode(',', $_POST['show_stats_post_chart_vbars']) : '';
		$show_stats_post_summary = $show_stats && !empty($_POST['show_stats_post_summary']) ? implode(',', $_POST['show_stats_post_summary']) : '';
		$show_stats_message_index_text = $show_stats ? $smcFunc['htmlspecialchars']($_POST['show_stats_message_index_text'], ENT_QUOTES) : '';
		$show_stats_message_index_chart_average = $show_stats && !empty($_POST['show_stats_message_index_chart_average']) && in_array('show', $_POST['show_stats_message_index_chart_average']) ? implode(',', $_POST['show_stats_message_index_chart_average']) : '';
		$show_stats_message_index_chart_hbars = $show_stats && !empty($_POST['show_stats_message_index_chart_hbars']) && in_array('show', $_POST['show_stats_message_index_chart_hbars']) ? implode(',', $_POST['show_stats_message_index_chart_hbars']) : '';
		$show_stats_message_index_chart_vbars = $show_stats && !empty($_POST['show_stats_message_index_chart_vbars']) && in_array('show', $_POST['show_stats_message_index_chart_vbars']) ? implode(',', $_POST['show_stats_message_index_chart_vbars']) : '';
		$show_stats_message_index_summary = $show_stats && !empty($_POST['show_stats_message_index_summary']) ? implode(',', $_POST['show_stats_message_index_summary']) : '';
		$show_stats_order_by = $show_stats && !empty($_POST['show_stats_order_by']) ? implode(',', $_POST['show_stats_order_by']) : '';
		$moderation_list_show = 'member' == $_POST['type'] && isset($_POST['moderation_list_show']) ? 1 : 0;
		if ($moderation_list_show)
		{
			$moderation_list_menu = isset($_POST['moderation_list_menu']) ? 1 : 0;
			$moderation_list_name = $_POST['moderation_list_name'];
			$moderation_list_order_by = !empty($_POST['moderation_list_order_by']) ? $_POST['moderation_list_order_by'] : 'cffpf.poster_time desc';
		}
		else
		{
			$moderation_list_menu = 0;
			$moderation_list_name = '';
			$moderation_list_order_by = '';
		}

		$show_group_name_filter = isset($_POST['show_group_name_filter']) ? 1 : 0;
		$show_group_name_search = isset($_POST['show_group_name_search']) ? 1 : 0;
		if ('filters' == $_POST['type'] && !empty($_POST['show_filters']))
		{
			$show_filters = implode(',', $_POST['show_filters']);
			$show_filters_operator_array = !empty($_POST['show_filters_operator']) ? $_POST['show_filters_operator'] : array();
			$show_filters_operator_array = array_intersect($show_filters_operator_array, $_POST['show_filters']);
			$show_filters_operator = !empty($show_filters_operator_array) ? implode(',', $show_filters_operator_array) : '';
		}
		else
		{
			$show_filters = '';
			$show_filters_operator = '';
		}

		if ('record' == $_POST['type'])
		{
			if (!cffpIsNotEmptyString($_POST['search_page_name_record']))
				$search_page_name = '';
			else
			{
				$search_page_name = $smcFunc['htmlspecialchars']($_POST['search_page_name_record'], ENT_QUOTES);
				$search_page_name = strtr($search_page_name, array(',' => ''));
				$search_page_name = trim($search_page_name);

				if (in_array($search_page_name, $context['params']['search_page_post_more_array']))
					fatal_lang_error('error_cffp_search_page_in_use');

				$request = $smcFunc['db_query']('', '
					SELECT id_group
					FROM {db_prefix}cffp_settings_groups
					WHERE type = {string:type}' . (!$context['gid'] ? '' : '
						AND id_group <> {int:id_group}'). '
						AND search_page_name = {string:search_page_name}',
					array(
						'type' => 'record',
						'id_group' => $context['gid'],
						'search_page_name' => $search_page_name,
					)
				);
				$num_rows = $smcFunc['db_num_rows']($request);
				$smcFunc['db_free_result']($request);
				if (0 < $num_rows)
					fatal_lang_error('error_cffp_search_page_in_use');
			}
			$search_page_order_by = $_POST['search_page_order_by_record'];
		}
		elseif (empty($_POST['search_page_post_default']))
		{
			$search_page_name_array = array();
			$search_page_order_by_array = array();

			foreach ($_POST['search_page_name_post'] as $k => $v)
			{
				// Clean, clean, clean...
				$v = $smcFunc['htmlspecialchars']($v, ENT_QUOTES);
				$v = strtr($v, array(',' => ''));
				$v = trim($v);

				if (!cffpIsNotEmptyString($v))
					continue;

				if (in_array($v, $search_page_name_array))
					continue;

				$request = $smcFunc['db_query']('', '
					SELECT id_group
					FROM {db_prefix}cffp_settings_groups
					WHERE type = {string:type}
						AND search_page_name = {string:search_page_name}',
					array(
						'type' => 'record',
						'search_page_name' => $v,
					)
				);
				$num_rows = $smcFunc['db_num_rows']($request);
				$smcFunc['db_free_result']($request);
				if (0 < $num_rows)
					fatal_lang_error('error_cffp_search_page_in_use_record');

				$search_page_name_array[] = $v;
				$search_page_order_by_array[] = $_POST['search_page_order_by_post'][$k];
			}

			if (!empty($search_page_name_array))
			{
				$search_page_name = implode(',', $search_page_name_array);
				$search_page_order_by = implode(',', $search_page_order_by_array);
			}
			else
			{
				$search_page_name = '';
				$search_page_order_by = 'm.poster_time desc';
			}
		}
		else
		{
			$search_page_name = '';
			$search_page_order_by = 'm.poster_time desc';
		}

		$show_sticky = 'filters' != $_POST['type'] ? $_POST['show_sticky'] : null;
		if (is_numeric($show_sticky))
			$db_type_show_sticky = 'int';
		else
		{
			$db_type_show_sticky = 'raw';
			$show_sticky = 'NULL';
		}
		$show_locked = 'filters' != $_POST['type'] ? $_POST['show_locked'] : null;
		if (is_numeric($show_locked))
			$db_type_show_locked = 'int';
		else
		{
			$db_type_show_locked = 'raw';
			$show_locked = 'NULL';
		}
		$show_first_post_topic = 'filters' != $_POST['type'] && isset($_POST['show_first_post_topic']) ? 1 : 0;
		$show_replies_topic = 'filters' != $_POST['type'] && isset($_POST['show_replies_topic']) ? 1 : 0;
		$show_in_topic = 'member' == $_POST['type'] && $show_first_post_topic && !$show_replies_topic ? $_POST['show_in_topic'] : 'no';
		$active = isset($_POST['active']) ? 1 : 0;

		$group_order = isset($_POST['group_order']) ? (int) $_POST['group_order'] : 0;

		if ('filters' != $_POST['type'])
		{
			$no_show_with_subject_prefix = array_map('trim', explode(',', $_POST['no_show_with_subject_prefix']));
			$no_show_with_subject_prefix = array_filter($no_show_with_subject_prefix, 'cffpIsNotEmptyString');
			$no_show_with_subject_prefix = array_unique($no_show_with_subject_prefix);
			$no_show_with_subject_prefix = $smcFunc['htmlspecialchars'](implode(',', $no_show_with_subject_prefix), ENT_QUOTES);
		}
		else
			$no_show_with_subject_prefix = '';

		$javascript = 'filters' != $_POST['type'] ? $_POST['javascript'] : '';

		if (empty($_POST['boards']))
			$boards = '';
		else
			$boards = in_array('all', $_POST['boards']) ? 'all' : implode(',', $_POST['boards']);
		if (is_numeric($_POST['child_boards']))
		{
			$db_type_child_boards = 'int';
			$child_boards = $_POST['child_boards'];
		}
		else
		{
			$db_type_child_boards = 'raw';
			$child_boards = 'NULL';
		}

		$permissions = !empty($_POST['permissions']) ? $_POST['permissions'] : array();
		$permissions_remove = array();
		switch ($_POST['type'])
		{
			case 'fields':
			{
				foreach ($context['params']['permissions_users'] as $pu => $txt_pu)
				{
					$permissions_remove[] = $pu . ':stats';
					$permissions_remove[] = $pu . ':add';
					$permissions_remove[] = $pu . ':remove';
				}
				break;
			}
			case 'record':
			{
				foreach ($context['params']['permissions_users'] as $pu => $txt_pu)
					$permissions_remove[] = $pu . ':stats';
				break;
			}
			case 'member':
			{
				$permissions_remove[] = 'owner:add';
				if (!$show_stats)
				{
					foreach ($context['params']['permissions_users'] as $pu => $txt_pu)
						$permissions_remove[] = $pu . ':stats';
				}
				break;
			}
			case 'filters':
			{
				foreach ($context['params']['permissions_users'] as $pu => $txt_pu)
				{
					$permissions_remove[] = $pu . ':stats';
					$permissions_remove[] = $pu . ':add';
					$permissions_remove[] = $pu . ':modify';
					$permissions_remove[] = $pu . ':remove';
					$permissions_remove[] = $pu . ':move';
				}
				$permissions_remove[] = 'owner:view';
				break;
			}
		}
		if (!empty($permissions_remove))
			$permissions = array_diff($permissions, $permissions_remove);
		$permissions = implode(',', $permissions);

		// Do the insertion/updates.
		if ($context['gid'])
		{
			$smcFunc['db_query']('', '
				UPDATE {db_prefix}cffp_settings_groups
				SET
					name = {string:name}, description = {string:description},
					group_order = {int:group_order},
					text_input_before = {string:text_input_before}, text_input = {string:text_input},
					text_output = {string:text_output},
					boards = {string:boards}, child_boards = {' . $db_type_child_boards . ':child_boards},
					type = {string:type},
					show_group_name_message_index = {int:show_group_name_message_index}, show_group_name_filter = {int:show_group_name_filter},
					show_group_name_search = {int:show_group_name_search},
					show_fields_name = {int:show_fields_name}, show_fields_set = {int:show_fields_set},
					show_empty_fields_post = {int:show_empty_fields_post}, show_empty_fields_message_index = {int:show_empty_fields_message_index},
					show_poster_data = {int:show_poster_data}, show_modified_data = {int:show_modified_data},
					show_actions_as = {string:show_actions_as},
					show_member_expanded = {int:show_member_expanded}, show_member_expanded_data = {int:show_member_expanded_data},
					show_member_expanded_data_recent = {int:show_member_expanded_data_recent}, show_member_expanded_new = {int:show_member_expanded_new},
					show_member_recent_at_top = {int:show_member_recent_at_top}, member_max_height = {int:member_max_height},
					member_one_value = {int:member_one_value},
					show_stats = {int:show_stats},
					show_stats_post_text = {string:show_stats_post_text},
					show_stats_post_chart_average = {string:show_stats_post_chart_average}, show_stats_post_chart_hbars = {string:show_stats_post_chart_hbars},
					show_stats_post_chart_vbars = {string:show_stats_post_chart_vbars}, show_stats_post_summary = {string:show_stats_post_summary},
					show_stats_message_index_text = {string:show_stats_message_index_text},
					show_stats_message_index_chart_average = {string:show_stats_message_index_chart_average}, show_stats_message_index_chart_hbars = {string:show_stats_message_index_chart_hbars},
					show_stats_message_index_chart_vbars = {string:show_stats_message_index_chart_vbars}, show_stats_message_index_summary = {string:show_stats_message_index_summary},
					show_stats_order_by = {string:show_stats_order_by},
					show_sticky = {' . $db_type_show_sticky . ':show_sticky}, show_locked = {' . $db_type_show_locked . ':show_locked},
					show_first_post_topic = {int:show_first_post_topic}, show_replies_topic = {int:show_replies_topic},
					no_show_with_subject_prefix = {string:no_show_with_subject_prefix}, show_in_topic = {string:show_in_topic},
					show_filters = {string:show_filters}, show_filters_operator = {string:show_filters_operator},
					javascript = {string:javascript},
					search_page_name = {string:search_page_name}, search_page_order_by = {string:search_page_order_by},
					counter_show = {int:counter_show}, counter_name = {string:counter_name},
					top_poster_show = {int:top_poster_show}, top_poster_name = {string:top_poster_name},
					profile_list_show = {int:profile_list_show}, profile_list_name = {string:profile_list_name},
					moderation_list_show = {int:moderation_list_show}, moderation_list_menu = {int:moderation_list_menu},
					moderation_list_name = {string:moderation_list_name}, moderation_list_order_by = {string:moderation_list_order_by},
					permissions = {string:permissions}, active = {int:active}
				WHERE id_group = {int:id_group}',
				array(
					'id_group' => $context['gid'],
					'name' => $_POST['name'],
					'description' => $_POST['description'],
					'group_order' => $group_order,
					'text_input_before' => $_POST['text_input_before'],
					'text_input' => $_POST['text_input'],
					'text_output' => $_POST['text_output'],
					'boards' => $boards,
					'child_boards' => $child_boards,
					'type' => $_POST['type'],
					'show_group_name_message_index' => $show_group_name_message_index,
					'show_group_name_filter' => $show_group_name_filter,
					'show_group_name_search' => $show_group_name_search,
					'show_fields_name' => $show_fields_name,
					'show_fields_set' => $show_fields_set,
					'show_empty_fields_post' => $show_empty_fields_post,
					'show_empty_fields_message_index' => $show_empty_fields_message_index,
					'show_poster_data' => $show_poster_data,
					'show_modified_data' => $show_modified_data,
					'show_actions_as' => $show_actions_as,
					'show_member_expanded' => $show_member_expanded,
					'show_member_expanded_data' => $show_member_expanded_data,
					'show_member_expanded_data_recent' => $show_member_expanded_data_recent,
					'show_member_expanded_new' => $show_member_expanded_new,
					'show_member_recent_at_top' => $show_member_recent_at_top,
					'member_max_height' => $member_max_height,
					'member_one_value' => $member_one_value,
					'show_stats' => $show_stats,
					'show_stats_post_text' => $show_stats_post_text,
					'show_stats_post_chart_average' => $show_stats_post_chart_average,
					'show_stats_post_chart_hbars' => $show_stats_post_chart_hbars,
					'show_stats_post_chart_vbars' => $show_stats_post_chart_vbars,
					'show_stats_post_summary' => $show_stats_post_summary,
					'show_stats_message_index_text' => $show_stats_message_index_text,
					'show_stats_message_index_chart_average' => $show_stats_message_index_chart_average,
					'show_stats_message_index_chart_hbars' => $show_stats_message_index_chart_hbars,
					'show_stats_message_index_chart_vbars' => $show_stats_message_index_chart_vbars,
					'show_stats_message_index_summary' => $show_stats_message_index_summary,
					'show_stats_order_by' => $show_stats_order_by,
					'show_sticky' => $show_sticky,
					'show_locked' => $show_locked,
					'show_first_post_topic' => $show_first_post_topic,
					'show_replies_topic' => $show_replies_topic,
					'no_show_with_subject_prefix' => $no_show_with_subject_prefix,
					'show_in_topic' => $show_in_topic,
					'show_filters' => $show_filters,
					'show_filters_operator' => $show_filters_operator,
					'javascript' => $javascript,
					'search_page_name' => $search_page_name,
					'search_page_order_by' => $search_page_order_by,
					'counter_show' => $counter_show,
					'counter_name' => $counter_name,
					'top_poster_show' => $top_poster_show,
					'top_poster_name' => $top_poster_name,
					'profile_list_show' => $profile_list_show,
					'profile_list_name' => $profile_list_name,
					'moderation_list_show' => $moderation_list_show,
					'moderation_list_menu' => $moderation_list_menu,
					'moderation_list_name' => $moderation_list_name,
					'moderation_list_order_by' => $moderation_list_order_by,
					'permissions' => $permissions,
					'active' => $active,
				)
			);

			// Fix old values with new settings.
			if (('member' == $context['params']['type'] && 'member' != $_POST['type']) || ('member' != $context['params']['type'] && 'member' == $_POST['type']))
			{
				$smcFunc['db_query']('', '
					DELETE FROM {db_prefix}cffp_fields
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
					)
				);

				if ('member' == $context['params']['type'] && 'member' != $_POST['type'])
				{
					$smcFunc['db_query']('', '
						UPDATE {db_prefix}cffp_settings_fields
						SET
							show_moderation_list_type = {string:no_column},
							show_moderation_list_width = {string:empty_string},
							show_moderation_list_name = {string:empty_string}
						WHERE id_group = {int:id_group}',
						array(
							'id_group' => $context['gid'],
							'no_column' => 'no',
							'empty_string' => '',
						)
					);
				}
			}

			if ('member' == $context['params']['type'] && 'member' == $_POST['type'] && !$context['params']['member_one_value'] && $member_one_value)
			{
				$request = $smcFunc['db_query']('', '
					SELECT id_msg, id_member, COUNT(*) AS num
					FROM {db_prefix}cffp_fields
					WHERE id_group = {int:id_group}
						AND id_member <> {int:guest}
					GROUP BY id_msg, id_member
					HAVING num > {int:one_value}',
					array(
						'id_group' => $context['gid'],
						'guest' => 0,
						'one_value' => 1,
					)
				);
				while ($row = $smcFunc['db_fetch_assoc']($request))
				{
					$smcFunc['db_query']('', '
						DELETE FROM {db_prefix}cffp_fields
						WHERE id_group = {int:id_group}
							AND id_msg = {int:id_msg}
							AND id_member = {int:id_member}',
						array(
							'id_group' => $context['gid'],
							'id_msg' => $row['id_msg'],
							'id_member' => $row['id_member'],
						)
					);
				}
				$smcFunc['db_free_result']($request);
			}

			if ('record' == $context['params']['type'] && 'record' != $_POST['type'])
			{
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}cffp_settings_fields
					SET
						show_record_list_type = {string:no_column},
						show_record_list_width = {string:empty_string},
						show_record_list_name = {string:empty_string}
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
						'no_column' => 'no',
						'empty_string' => '',
					)
				);
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}cffp_settings_groups
					SET
						template_quick_input_regex = {string:empty_string},
						template_quick_input_params = {string:empty_string},
						template_quick_input_help = {string:empty_string},
						template_quick_input_show_modify = {int:empty_int}
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
						'empty_string' => '',
						'empty_int' => 0,
					)
				);
				$smcFunc['db_query']('', '
					DELETE FROM {db_prefix}cffp_count_groups
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
					)
				);
			}

			if ('record' != $context['params']['type'] && 'record' == $_POST['type'])
				cffpUpdateCountGroupAllMembers($context['gid']);

			if ('filters' != $context['params']['type'] && 'filters' == $_POST['type'])
			{
				$smcFunc['db_query']('', '
					DELETE FROM {db_prefix}cffp_fields
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
					)
				);
				$smcFunc['db_query']('', '
					DELETE FROM {db_prefix}cffp_settings_fields
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
					)
				);
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}cffp_settings_groups
					SET
						template_quick_input_regex = {string:empty_string},
						template_quick_input_params = {string:empty_string},
						template_quick_input_help = {string:empty_string},
						template_quick_input_show_modify = {int:empty_int},
						template_text = {string:empty_string},
						template_message_index_text = {string:empty_string}
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
						'empty_string' => '',
						'empty_int' => 0,
					)
				);
			}

			if ($context['params']['show_stats'] && !$show_stats)
			{
				$smcFunc['db_query']('', '
					DELETE FROM {db_prefix}cffp_stats
					WHERE id_group = {int:id_group}',
					array(
						'id_group' => $context['gid'],
					)
				);
			}
			elseif (!$context['params']['show_stats'] && $show_stats)
				cffpUpdateStats($context['gid'], 0, true);

			$id_group = $context['gid'];
		}
		else
		{
			$smcFunc['db_insert']('',
				'{db_prefix}cffp_settings_groups',
				array(
					'name' => 'string', 'description' => 'string',
					'group_order' => 'int',
					'text_input_before' => 'string', 'text_input' => 'string', 'text_output' => 'string',
					'boards' => 'string', 'child_boards' => $db_type_child_boards,
					'type' => 'string',
					'show_group_name_message_index' => 'int', 'show_group_name_filter' => 'int',
					'show_group_name_search' => 'int',
					'show_fields_name' => 'int', 'show_fields_set' => 'int',
					'show_empty_fields_post' => 'int', 'show_empty_fields_message_index' => 'int',
					'show_poster_data' => 'int', 'show_modified_data' => 'int',
					'show_actions_as' => 'string',
					'show_member_expanded' => 'int', 'show_member_expanded_data' => 'int',
					'show_member_expanded_data_recent' => 'int', 'show_member_expanded_new' => 'int',
					'show_member_recent_at_top' => 'int', 'member_max_height' => 'int',
					'member_one_value' => 'int',
					'show_stats' => 'int',
					'show_stats_post_text' => 'string',
					'show_stats_post_chart_average' => 'string', 'show_stats_post_chart_hbars' => 'string',
					'show_stats_post_chart_vbars' => 'string', 'show_stats_post_summary' => 'string',
					'show_stats_message_index_text' => 'string',
					'show_stats_message_index_chart_average' => 'string', 'show_stats_message_index_chart_hbars' => 'string',
					'show_stats_message_index_chart_vbars' => 'string', 'show_stats_message_index_summary' => 'string',
					'show_stats_order_by' => 'string',
					'show_sticky' => $db_type_show_sticky, 'show_locked' => $db_type_show_locked,
					'show_first_post_topic' => 'int', 'show_replies_topic' => 'int',
					'no_show_with_subject_prefix' => 'string', 'show_in_topic' => 'string',
					'show_filters' => 'string', 'show_filters_operator' => 'string',
					'javascript' => 'string',
					'search_page_name' => 'string', 'search_page_order_by' => 'string',
					'counter_show' => 'int', 'counter_name' => 'string',
					'top_poster_show' => 'int', 'top_poster_name' => 'string',
					'profile_list_show' => 'int', 'profile_list_name' => 'string',
					'moderation_list_show' => 'int', 'moderation_list_menu' => 'int',
					'moderation_list_name' => 'string', 'moderation_list_order_by' => 'string',
					'permissions' => 'string', 'active' => 'int',
				),
				array(
					$_POST['name'], $_POST['description'],
					$group_order,
					$_POST['text_input_before'], $_POST['text_input'], $_POST['text_output'],
					$boards, $child_boards,
					$_POST['type'],
					$show_group_name_message_index, $show_group_name_filter,
					$show_group_name_search,
					$show_fields_name, $show_fields_set,
					$show_empty_fields_post, $show_empty_fields_message_index,
					$show_poster_data, $show_modified_data,
					$show_actions_as,
					$show_member_expanded, $show_member_expanded_data,
					$show_member_expanded_data_recent, $show_member_expanded_new,
					$show_member_recent_at_top, $member_max_height,
					$member_one_value,
					$show_stats,
					$show_stats_post_text,
					$show_stats_post_chart_average, $show_stats_post_chart_hbars,
					$show_stats_post_chart_vbars, $show_stats_post_summary,
					$show_stats_message_index_text,
					$show_stats_message_index_chart_average, $show_stats_message_index_chart_hbars,
					$show_stats_message_index_chart_vbars, $show_stats_message_index_summary,
					$show_stats_order_by,
					$show_sticky, $show_locked,
					$show_first_post_topic, $show_replies_topic,
					$no_show_with_subject_prefix, $show_in_topic,
					$show_filters, $show_filters_operator,
					$javascript,
					$search_page_name, $search_page_order_by,
					$counter_show, $counter_name,
					$top_poster_show, $top_poster_name,
					$profile_list_show, $profile_list_name,
					$moderation_list_show, $moderation_list_menu,
					$moderation_list_name, $moderation_list_order_by,
					$permissions, $active,
				),
				array('id_group')
			);
			$id_group = $smcFunc['db_insert_id']('{db_prefix}cffp_settings_groups', 'id_group');
		}

		// Fix old values with new settings.
		if (!empty($id_group) && cffpIsNotEmptyString($search_page_name))
		{
			$search_page_name_array = explode(',', $search_page_name);
			$search_page_order_by_array = explode(',', $search_page_order_by);
			foreach ($search_page_name_array as $k => $v)
			{
				$request = $smcFunc['db_query']('', '
					SELECT id_group, search_page_name, search_page_order_by
					FROM {db_prefix}cffp_settings_groups
					WHERE id_group <> {int:id_group}
						AND type IN ({array_string:type})
						AND (search_page_name = {string:search_page_name1} OR search_page_name LIKE {string:search_page_name2} OR search_page_name LIKE {string:search_page_name3} OR search_page_name LIKE {string:search_page_name4})',
					array(
						'id_group' => $id_group,
						'type' => array('fields', 'member', 'filters'),
						'search_page_name1' => $v,
						'search_page_name2' => $v . ',%',
						'search_page_name3' => '%,' . $v,
						'search_page_name4' => '%,' . $v . ',%',
					)
				);
				while ($row = $smcFunc['db_fetch_assoc']($request))
				{
					if (strpos($row['search_page_name'], ','))
					{
						$search_page_name_array2 = explode(',', $row['search_page_name']);
						$search_page_order_by_array2 = explode(',', $row['search_page_order_by']);
						foreach ($search_page_name_array2 as $k2 => $v2)
						{
							if ($v2 == $v)
							{
								if ($search_page_order_by_array2[$k2] != $search_page_order_by_array[$k])
								{
									$search_page_order_by_array2[$k2] = $search_page_order_by_array[$k];
									$smcFunc['db_query']('', '
										UPDATE {db_prefix}cffp_settings_groups
										SET search_page_order_by = {string:search_page_order_by}
										WHERE id_group = {int:id_group}',
										array(
											'id_group' => $row['id_group'],
											'search_page_order_by' => implode(',', $search_page_order_by_array2),
										)
									);
								}
								break; // Exit foreach.
							}
						}
					}
					elseif ($row['search_page_order_by'] != $search_page_order_by_array[$k])
					{
						$smcFunc['db_query']('', '
							UPDATE {db_prefix}cffp_settings_groups
							SET search_page_name = {string:search_page_name}, search_page_order_by = {string:search_page_order_by}
							WHERE id_group = {int:id_group}',
							array(
								'id_group' => $row['id_group'],
								'search_page_name' => $v,
								'search_page_order_by' => $search_page_order_by_array[$k],
							)
						);
					}
				}
				$smcFunc['db_free_result']($request);
			}
		}
	}
	// Deleting?
	elseif (isset($_GET['delete']) && $context['gid'])
	{
		checkSession('get');

		// Delete statistics of group.
		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}cffp_stats
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);

		// Delete group on posts.
		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}cffp_fields
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);

		// Delete count of group.
		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}cffp_count_groups
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);

		// Delete columns of fields on posts.
		$request = $smcFunc['db_query']('', '
			SELECT id_column
			FROM {db_prefix}cffp_settings_fields
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);
		while ($row = $smcFunc['db_fetch_assoc']($request))
			cffpRemoveColumn($context['gid'], $row['id_column']);
		$smcFunc['db_free_result']($request);

		// Delete setting fields of group.
		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}cffp_settings_fields
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);

		// Finally delete setting group.
		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}cffp_settings_groups
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);
	}

	if (isset($_GET['delete']) || isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=admin;area=modsettings;sa=cffp' . $context['params_url']);
	}
}

// Admin area settings (edit template of group).
function cffpSettingsEditTemplate($return_config = false)
{
	global $txt, $context, $smcFunc;

	if ($return_config)
		return array();

	loadTemplate('CustomFieldFilterPost');

	// Sort out the context!
	$context['gid'] = isset($_GET['gid']) ? (int) $_GET['gid'] : 0;
	$context['settings_title'] = '';
	$start = isset($_GET['start']) ? (int) $_GET['start'] : 0;
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'group_order';
	$context['params_url'] = ';start=' . $start . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context[$context['admin_menu_name']]['current_subsection'] = 'cffp';
	$context['sub_template'] = 'cffp_settings_edit_template';

	if ($context['gid'])
	{
		$request = $smcFunc['db_query']('', '
			SELECT
				name, type,
				template_quick_input_regex, template_quick_input_params,
				template_quick_input_help, template_quick_input_show_modify,
				template_text, template_type,
				template_message_index_text, template_message_index_type
			FROM {db_prefix}cffp_settings_groups
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $context['gid'],
			)
		);
		$group = $smcFunc['db_fetch_assoc']($request);
		$smcFunc['db_free_result']($request);

		$group['type_txt'] = $txt['cffp_grouptype_' . $group['type']];

		ob_start();
		cffpSettingsShowFields($context['gid'], '', 'template');
		$list = ob_get_contents();
		ob_end_clean();

		$context['params'] = array(
			'group' => $group,
			'fields' => $list,
			'template_quick_input_regex' => $smcFunc['htmlspecialchars']($group['template_quick_input_regex'], ENT_QUOTES),
			'template_quick_input_params' => $group['template_quick_input_params'],
			'template_quick_input_help' => $group['template_quick_input_help'],
			'template_quick_input_show_modify' => $group['template_quick_input_show_modify'],
			'template_type' => $group['template_type'],
			'template_text' => $smcFunc['htmlspecialchars']($group['template_text'], ENT_QUOTES),
			'template_message_index_type' => $group['template_message_index_type'],
			'template_message_index_text' => $smcFunc['htmlspecialchars']($group['template_message_index_text'], ENT_QUOTES),
		);

		$context['settings_title'] = empty($context['params']['template_quick_input_regex']) && empty($context['params']['template_text']) && empty($context['params']['template_message_index_text']) ? $txt['cffp_add_templates'] : $txt['cffp_edit_templates'];
	}

	// Others data.
	$template_type_array = array('html', 'bbc', 'php');
	$context['params']['template_type_array'] = array();
	foreach ($template_type_array as $template_type)
		$context['params']['template_type_array'][$template_type] = $txt['cffp_template_type_' . $template_type];

	// Are we saving?
	if (isset($_POST['save']))
	{
		checkSession('get');

		// Validate inputs.
		if ('record' == $context['params']['group']['type'])
		{
			$template_quick_input_regex = $_POST['template_quick_input_regex'];

			$template_quick_input_params = array_map('trim', preg_split('~[\n\r]+~', $_POST['template_quick_input_params']));
			$template_quick_input_params = array_filter($template_quick_input_params, 'cffpIsNotEmptyString');
			$can_be_js = false;
			foreach ($template_quick_input_params as $k => $param)
			{
				if (is_numeric($param))
					$can_be_js = $param ? true : false;
				elseif ($can_be_js)
					$can_be_js = false;
				else
					unset($template_quick_input_params[$k]);
			}
			$template_quick_input_params = implode("\n", $template_quick_input_params);

			$template_quick_input_help = $smcFunc['htmlspecialchars']($_POST['template_quick_input_help'], ENT_QUOTES);
			$template_quick_input_show_modify = isset($_POST['template_quick_input_show_modify']) ? 1 : 0;
		}
		else
		{
			$template_quick_input_regex = '';
			$template_quick_input_params = '';
			$template_quick_input_help = '';
			$template_quick_input_show_modify = 0;
		}

		// Do the updates.
		if ($context['gid'])
		{
			$smcFunc['db_query']('', '
				UPDATE {db_prefix}cffp_settings_groups
				SET
					template_quick_input_regex = {string:template_quick_input_regex},
					template_quick_input_params = {string:template_quick_input_params},
					template_quick_input_help = {string:template_quick_input_help},
					template_quick_input_show_modify = {int:template_quick_input_show_modify},
					template_type = {string:template_type},
					template_text = {string:template_text},
					template_message_index_type = {string:template_message_index_type},
					template_message_index_text = {string:template_message_index_text}
				WHERE id_group = {int:id_group}',
				array(
					'id_group' => $context['gid'],
					'template_quick_input_regex' => $template_quick_input_regex,
					'template_quick_input_params' => $template_quick_input_params,
					'template_quick_input_help' => $template_quick_input_help,
					'template_quick_input_show_modify' => $template_quick_input_show_modify,
					'template_type' => $_POST['template_type'],
					'template_text' => $_POST['template_text'],
					'template_message_index_type' => $_POST['template_message_index_type'],
					'template_message_index_text' => $_POST['template_message_index_text'],
				)
			);
		}
	}

	if (isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=admin;area=modsettings;sa=cffp' . $context['params_url']);
	}
}

// Admin area settings (show).
function cffpSettingsShow($return_config = false)
{
	global $txt, $scripturl, $context, $sourcedir;

	if ($return_config)
		return array();

	loadTemplate('CustomFieldFilterPost');
	require_once ($sourcedir . '/Subs-List.php');

	$context['sub_template'] = 'cffp_settings_show';
	$context['settings_title'] = $txt['cffp'];

	$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;
	$sort = isset($_REQUEST['sort']) ? $_REQUEST['sort'] : 'group_order';
	$params_url = ';start=' . $start . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : '');

	$listOptions = array(
		'id' => 'cffp_settings_show_groups',
		'title' => $txt['cffp_group_of_fields'],
		'base_href' => $scripturl . '?action=admin;area=modsettings;sa=cffp',
		'default_sort_col' => 'group_order',
		'no_items_label' => $txt['cffp_none_group'],
		'items_per_page' => 25,
		'get_items' => array(
			'function' => 'cffpGetListSettingsGroups',
		),
		'get_count' => array(
			'function' => 'cffpGetListSettingsGroupsSize',
		),
		'columns' => array(
			'group_order' => array(
				'header' => array(
					'value' => $txt['cffp_order'],
					'style' => 'width: 5%; text-align: left;',
				),
				'data' => array(
					'db' => 'group_order',
					'style' => 'text-align: center;',
				),
				'sort' => array(
					'default' => 'group_order',
					'reverse' => 'group_order DESC',
				),
			),
			'id_group' => array(
				'header' => array(
					'value' => $txt['cffp_id'],
					'style' => 'width: 5%; font-size: smaller;',
				),
				'data' => array(
					'db' => 'id_group',
					'style' => 'text-align: center;',
					'class' => 'smalltext',
				),
			),
			'name' => array(
				'header' => array(
					'value' => $txt['cffp_name'],
					'style' => 'width: 15%; text-align: left;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						$text_name = $rowData[\'name\'];
						if (cffpIsNotEmptyString($rowData[\'description\']))
							$text_name .= \'<br />/<br /><span class="smalltext">\' . nl2br($rowData[\'description\']) . \'</span>\';

						return $text_name;
					'),
				),
				'sort' => array(
					'default' => 'name',
					'reverse' => 'name DESC',
				),
			),
			'type' => array(
				'header' => array(
					'value' => $txt['cffp_type'],
					'style' => 'width: 15%; text-align: left;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $txt;

						$textKey = \'cffp_grouptype_\' . $rowData[\'type\'];
						return isset($txt[$textKey]) ? $txt[$textKey] : $textKey;
					'),
				),
				'sort' => array(
					'default' => 'type',
					'reverse' => 'type DESC',
				),
			),
			'active' => array(
				'header' => array(
					'value' => $txt['cffp_active'],
					'style' => 'width: 5%;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $txt;

						return $rowData[\'active\'] ? $txt[\'yes\'] : $txt[\'no\'];
					'),
					'style' => 'text-align: center;',
				),
				'sort' => array(
					'default' => 'active DESC',
					'reverse' => 'active',
				),
			),
			'fields' => array(
				'header' => array(
					'value' => $txt['cffp_fields'],
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $txt, $scripturl, $context;

						if (\'filters\' == $rowData[\'type\'])
						{
							ob_start();
							cffpSettingsShowFilters($rowData[\'show_filters\']);
							$list = ob_get_contents();
							ob_end_clean();

							$text = \'\';
						}
						else
						{
							ob_start();
							cffpSettingsShowFields($rowData[\'id_group\'], \'' . $params_url . '\');
							$list = ob_get_contents();
							ob_end_clean();

							$text = \'<a href="\' . $scripturl . \'?action=admin;area=modsettings;sa=cffpedittemplate;gid=\' . $rowData[\'id_group\'] . \'' . $params_url . ';\' . $context[\'session_var\'] . \'=\' . $context[\'session_id\'] . \'">\' . (empty($rowData[\'template_quick_input_regex\']) && empty($rowData[\'template_text\']) && empty($rowData[\'template_message_index_text\']) ? $txt[\'cffp_add_templates\'] : $txt[\'cffp_edit_templates\']) . \'</a>\';
						}

						return \'<div style="max-height: 200px; overflow: scroll;">\' . $list . \'</div>\' . $text;
					'),
					'style' => 'text-align: right;',
				),
			),
			'option_modify' => array(
				'header' => array(
					'style' => 'width: 10%;',
				),
				'data' => array(
					'sprintf' => array(
						'format' => '<a href="' . $scripturl . '?action=admin;area=modsettings;sa=cffpeditgroup;gid=%1$s' . $params_url . '">' . $txt['modify'] . '</a>',
						'params' => array(
							'id_group' => false,
						),
					),
					'style' => 'text-align: center;',
				),
			),
			'option_delete' => array(
				'header' => array(
					'style' => 'width: 10%;',
				),
				'data' => array(
					'sprintf' => array(
						'format' => '<a href="' . $scripturl . '?action=admin;area=modsettings;sa=cffpeditgroup;delete=1;gid=%1$s' . $params_url . ';' . $context['session_var'] . '=' . $context['session_id'] . '" onclick="return confirm(\'' . addslashes($txt['cffp_edit_delete_group_sure']) . '\');">' . $txt['delete'] . '</a>',
						'params' => array(
							'id_group' => false,
						),
					),
					'style' => 'text-align: center;',
				),
			),
		),
		'form' => array(
			'href' => $scripturl . '?action=admin;area=modsettings;sa=cffpeditgroup' . $params_url,
			'name' => 'cffpGroup',
		),
		'additional_rows' => array(
			array(
				'position' => 'below_table_data',
				'value' => '<input type="submit" name="new" value="' . $txt['cffp_make_new_group'] . '" class="button_submit" />',
				'style' => 'text-align: right;',
			),
		),
	);
	createList($listOptions);
}

// Admin area settings (show fields of a group).
function cffpSettingsShowFields($id_group, $params_url, $type = '')
{
	global $txt, $scripturl, $context, $sourcedir;

	if (empty($id_group))
		return false;

	loadTemplate('GenericList');

	$headers = array(
		array(
			'label' => $txt['cffp_order'],
			'style' => 'width: 5%; font-size: smaller;',
		),
		array(
			'label' => $txt['cffp_id'],
			'style' => 'width: 5%; font-size: smaller;',
		),
		array(
			'label' => $txt['cffp_name'],
			'style' => 'width: ' . (empty($type) ? 15 : 25) . '%; text-align: left; font-size: smaller;',
		),
		array(
			'label' => $txt['cffp_type'],
			'style' => 'width: 15%; text-align: left; font-size: smaller;',
		),
		array(
			'label' => $txt['cffp_more_info'],
			'style' => 'text-align: left; font-size: smaller;',
		),
	);
	if (empty($type))
	{
		$headers[] = array(
			'label' => $txt['cffp_active'],
			'style' => 'width: 5%; font-size: smaller;',
		);
		$headers[] = array(
			'style' => 'width: 8%;',
		);
		$headers[] = array(
			'style' => 'width: 8%;',
		);
	}

	$rows = array();
	if (!empty($type) && isset($context['cffp']['settings_groups'][$id_group]))
		$list = $context['cffp']['settings_groups'][$id_group]['settings_fields'];
	else
		$list = cffpGetListSettingsFields($id_group, !empty($type));
	foreach ($list as $rowData)
	{
		$text_name = $rowData['name'];
		if (cffpIsNotEmptyString($rowData['field_desc']))
			$text_name .= '<br /><span class="smalltext">' . $rowData['field_desc'] . '</span>';
		if (cffpIsNotEmptyString($rowData['description']))
			$text_name .= '<br />/<br /><span class="smalltext">' . nl2br($rowData['description']) . '</span>';

		$textKey = 'cffp_fieldtype_' . $rowData['type'];
		$row = array(
			array(
				'value' => $rowData['field_order'],
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			),
			array(
				'value' => $rowData['id_field'] . (!empty($type) && $rowData['show_message_index'] ? ' <span class="smalltext">(*)</span>' : ''),
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			),
			array(
				'value' => $text_name,
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
			array(
				'value' => isset($txt[$textKey]) ? $txt[$textKey] : $textKey,
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
			array(
				'value' => cffpGetMoreInfoField($rowData),
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
		);
		if (empty($type))
		{
			$row[] = array(
				'value' => $rowData['active'] ? $txt['yes'] : $txt['no'],
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			);
			$row[] = array(
				'value' => '<a href="' . $scripturl . '?action=admin;area=modsettings;sa=cffpeditfield;gid=' . $id_group . ';fid=' . $rowData['id_field'] . $params_url . ';' . $context['session_var'] . '=' . $context['session_id'] . '">' . $txt['modify'] . '</a>',
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			);
			$row[] = array(
				'value' => '<a href="' . $scripturl . '?action=admin;area=modsettings;sa=cffpeditfield;delete=1;gid=' . $id_group . ';fid=' . $rowData['id_field'] . $params_url . ';' . $context['session_var'] . '=' . $context['session_id'] . '" onclick="return confirm(\'' . addslashes($txt['cffp_edit_delete_field_sure']) . '\');">' . $txt['delete'] . '</a>',
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			);
		}
		$rows[] = $row;
	}

	$additional_rows = array();
	if (empty($type))
	{
		$additional_rows['below_table_data'] = array(
			array(
				'value' => '<a href="' . $scripturl . '?action=admin;area=modsettings;sa=cffpeditfield;gid=' . $id_group . $params_url . ';' . $context['session_var'] . '=' . $context['session_id'] . '">' . $txt['cffp_make_new_field'] . '</a>',
				'style' => 'text-align: right;',
				'class' => 'smalltext',
			),
		);
	}

	$context['cffp_settings_show_fields'] = array(
		'no_items_label' => $txt['cffp_none_field'],
		'num_columns' => count($headers),
		'headers' => $headers,
		'rows' => $rows,
		'additional_rows' => $additional_rows,
	);

	template_show_list('cffp_settings_show_fields');
}

// Admin area settings (show filters of a group).
function cffpSettingsShowFilters($filters)
{
	global $txt, $context, $modSettings;

	loadTemplate('GenericList');

	if (!empty($filters))
	{
		$settings_filters = cffpGetSettingsPredefinedFilters();
		$filters = explode(',', $filters);
	}
	else
		$filters = array();

	$headers = array(
		array(
			'label' => $txt['cffp_show_filters'],
			'style' => 'text-align: left; font-size: smaller;',
		),
		array(
		),
	);

	$rows = array();
	foreach ($filters as $filter)
	{
		$note_array = array();
		if ('sticky' == $filter && !$modSettings['enableStickyTopics'])
			$note_array[] = $txt['cffp_show_filters_sticky_disabled'];
		$note = !empty($note_array) ? ' <div class="smalltext">' . implode('<br />', $note_array) . '</div>' : '';

		$rows[] = array(
			array(
				'value' => $settings_filters[$filter]['name'][0] . (!empty($settings_filters[$filter]['field'][0]) ? $note : ''),
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
			array(
				'value' => $settings_filters[$filter]['name'][1] . (!empty($settings_filters[$filter]['field'][1]) ? $note : ''),
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
		);
	}

	$context['cffp_settings_show_fields'] = array(
		'no_items_label' => $txt['cffp_none_field'],
		'num_columns' => count($headers),
		'headers' => $headers,
		'rows' => $rows,
	);

	template_show_list('cffp_settings_show_fields');
}

?>